------------------------------------------------------------------------------
--	FILE:	 LargestEarth.Lua
--  Convertion and upscaling of Civ IV Genghis Kai's GEM
--  Made for Civ VI by Gedemon (2016)
------------------------------------------------------------------------------

include "MapEnums"
include "MapUtilities"
include "MountainsCliffs"
include "RiversLakes"
include "FeatureGenerator"
include "TerrainGenerator"
include "NaturalWonderGenerator"
include "ResourceGenerator"
include "AssignStartingPlots"

local mapName = MapConfiguration.GetValue("MapName")
print("Loading "..tostring(mapName).." script")

local g_iW = 230
local g_iH = 116
local g_iFlags = {}
local g_continentsFrac = nil

------------------------------------------------------------------------------
-- The application side will call GetMapScriptInfo directly to request
-- information about the map script.
-------------------------------------------------------------------------------
function GetMapInitData(worldSize)
	return {
		Width = g_iW,
		Height = g_iH,
		WrapX = true,
		WrapY = false,
	};      
end


-------------------------------------------------------------------------------
function GenerateMap()
	print("Calling Map Generator");
	-- Everything has been moved to the modded AssignStartingPlots in the "override" folder, search for "YnAMP" string in that file
	GenerateImportedMap(GetMap(), GetCiv6DataToConvert(), GetNaturalWonders(), g_iW, g_iH)
	
	-- Handle Nile delta flow direction (North)
	for x = 38, 41 do
		for y = 58, 62 do
			local plot = Map.GetPlot(x,y)
			if plot then
				MakeRiverFlowToNorth(plot)			
			end
		end
	end
	
	-- Handle source of the Nile flow direction	
	for x = 40, 41 do
		for y = 36, 38 do
			local plot = Map.GetPlot(x,y)
			if plot then
				MakeRiverFlowToNorth(plot)			
			end
		end
	end
end

function GetNaturalWonders()
	local NaturalWonders = {}
	---[[
	NaturalWonders[GameInfo.Features["FEATURE_BARRIER_REEF"].Index] 	 = { X = 134, Y = 28}	-- 2 plots, coast, 1st plot is SOUTHEAST	
	NaturalWonders[GameInfo.Features["FEATURE_CLIFFS_DOVER"].Index] 	 = { X = 15, Y = 89}	-- 2 plots, hills on coast, 1st plot is WEST	
	NaturalWonders[GameInfo.Features["FEATURE_CRATER_LAKE"].Index] 		 = { X = 176, Y = 80}	
	NaturalWonders[GameInfo.Features["FEATURE_DEAD_SEA"].Index] 		 = { X = 47, Y = 62}	-- 2 plots, flat desert surrounded by desert, 1st plot is SOUTHWEST	
	NaturalWonders[GameInfo.Features["FEATURE_EVEREST"].Index] 			 = { X = 87, Y = 67}	-- 3 plots, mountains, 1st plot is WEST	
	NaturalWonders[GameInfo.Features["FEATURE_GALAPAGOS"].Index] 		 = { X = 192, Y = 44}   -- 2 plots, coast, surrounded by coast, 1st plot is SOUTHEAST
	NaturalWonders[GameInfo.Features["FEATURE_KILIMANJARO"].Index] 		 = { X = 45, Y = 36}	
	NaturalWonders[GameInfo.Features["FEATURE_PANTANAL"].Index] 		 = { X = 212, Y = 34}	-- 4 plots, flat grass/plains without features, 1st plot is SOUTH-WEST	
	NaturalWonders[GameInfo.Features["FEATURE_PIOPIOTAHI"].Index] 		 = { X = 143, Y = 4}	-- 3 plots, flat grass near coast, 1st plot is WEST	
	NaturalWonders[GameInfo.Features["FEATURE_TORRES_DEL_PAINE"].Index]  = { X = 205, Y = 8}	-- 2 plots EAST-WEST without features, 1st plot is WEST	
	NaturalWonders[GameInfo.Features["FEATURE_TSINGY"].Index] 			 = { X = 47, Y = 22}	
	NaturalWonders[GameInfo.Features["FEATURE_YOSEMITE"].Index] 		 = { X = 176, Y = 76}	-- 2 plots EAST-WEST, flat tundra/plains without features, 1st plot is WEST
	
	if GameInfo.Features["FEATURE_EYJAFJALLAJOKULL"] then -- Vikings DLC is loaded
	NaturalWonders[GameInfo.Features["FEATURE_EYJAFJALLAJOKULL"].Index]	= { X = 3, Y = 104}		-- 2 plots, use YOSEMITE rule
	NaturalWonders[GameInfo.Features["FEATURE_LYSEFJORDEN"].Index] 		= { X = 23, Y = 100}	-- 3 plots, , flat grass near coast, 1st plot is EAST
	NaturalWonders[GameInfo.Features["FEATURE_GIANTS_CAUSEWAY"].Index] 	= { X = 7, Y = 97}		-- 2 plots, one on coastal land and one in water, 1st plot is land, SOUTHEAST 
	end
	--]]
	return NaturalWonders
end

function GetCiv6DataToConvert()
	-- Map data is exported from Civ6 WB, cliffs are included in MapToConvert
	return {}
end


function GetMap()

	local MapToConvert = {}
	for i = 0, g_iW - 1, 1 do
		MapToConvert[i] = {}
	end
	
	-- Map Data (from Civ6 WB)
	-- MapToConvert[x][y] = {civ6TerrainType, civ6FeatureTypes, civ6ContinentType, {{IsNEOfRiver, flow}, {IsWOfRiver, flow}, {IsNWOfRiver, flow}}, {Civ6ResourceType, num}, {IsNEOfCliff, IsWOfCliff, IsNWOfCliff} }

 MapToConvert[0][0]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][0]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][0]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][0]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][0]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][0]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][0]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][0]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][0]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][0]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][0]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][0]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][0]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][0]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][0]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][0]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][0]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][0]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][0]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][0]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][0]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][0]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][0]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][0]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][0]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][0]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][0]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][0]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][0]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][0]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][1]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][1]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][1]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][1]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][1]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][1]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][1]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][1]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][1]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][1]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][1]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[11][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[12][1]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][1]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][1]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][1]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][1]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][1]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][1]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][1]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][1]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][1]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][1]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][1]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][1]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][1]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][1]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][1]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][1]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][1]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][1]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][1]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][1]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][1]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][1]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][1]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][1]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][1]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][1]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][1]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][1]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[206][1]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][1]={10,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[208][1]={10,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[209][1]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[210][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][1]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][1]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[219][1]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][1]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][1]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][1]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][1]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][1]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][1]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][1]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][1]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][2]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][2]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][2]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][2]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][2]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][2]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][2]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][2]={13,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[10][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[12][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][2]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][2]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][2]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][2]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][2]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][2]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[75][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[76][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[77][2]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[79][2]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[80][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[81][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[82][2]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][2]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[88][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[89][2]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[90][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[91][2]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][2]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][2]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[146][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][2]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][2]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][2]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][2]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][2]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][2]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][2]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][2]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][2]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][2]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][2]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[207][2]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][2]={9,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[210][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[211][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][2]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][2]={12,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[222][2]={13,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[223][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][2]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][2]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][2]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][3]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[3][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[4][3]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][3]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][3]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[9][3]={12,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][3]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][3]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][3]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[75][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[79][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[80][3]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][3]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[88][3]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][3]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][3]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][3]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][3]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][3]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][3]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][3]={9,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[206][3]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][3]={9,-1,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,1,0}}
 MapToConvert[208][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][3]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][3]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[222][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[223][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][3]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][3]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[229][3]={9,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][4]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[5][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][4]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][4]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][4]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[132][4]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][4]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][4]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][4]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][4]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][4]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][4]={9,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[206][4]={9,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][4]={9,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[208][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][4]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][4]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][4]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[0][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][5]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][5]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][5]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][5]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][5]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][5]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][5]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][5]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][5]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][5]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][5]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][5]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][5]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][5]={4,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[132][5]={4,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][5]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][5]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][5]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][5]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][5]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[147][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[205][5]={9,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][5]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][5]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[208][5]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[209][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[214][5]={9,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,0}}
 MapToConvert[215][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][5]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][5]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][6]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][6]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][6]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][6]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][6]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][6]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[132][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[133][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][6]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][6]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][6]={0,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[147][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][6]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[206][6]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][6]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][6]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[210][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][6]={3,-1,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][6]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[216][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][6]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][6]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][7]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][7]={4,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][7]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][7]={9,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][7]={9,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][7]={11,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][7]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][7]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][7]={3,-1,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][7]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][7]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[33][8]={0,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[34][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][8]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][8]={2,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][8]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][8]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][8]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][8]={11,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][8]={11,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][8]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][8]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][8]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][8]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][8]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][9]={0,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[32][9]={1,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[33][9]={1,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][9]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][9]={4,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][9]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[37][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[130][9]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[131][9]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[132][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][9]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][9]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[136][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][9]={1,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][9]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][9]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[204][9]={9,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][9]={9,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][9]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][9]={4,-1,10,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[208][9]={3,3,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[209][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][9]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][9]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][10]={1,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][10]={0,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][10]={0,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][10]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][10]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][10]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[38][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][10]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][10]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][10]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][10]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][10]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][10]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][10]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][10]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][10]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][10]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[205][10]={9,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][10]={11,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][10]={4,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][10]={4,3,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][10]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[210][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][10]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][10]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][11]={0,-1,9,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[32][11]={1,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][11]={0,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[34][11]={4,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][11]={0,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[36][11]={0,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][11]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][11]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][11]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][11]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][11]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][11]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[117][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][11]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][11]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][11]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][11]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][11]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][11]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][11]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][11]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][11]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][11]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][11]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[204][11]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][11]={11,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][11]={4,3,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][11]={4,3,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][11]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][11]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[210][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][11]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][11]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[31][12]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[32][12]={0,-1,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[33][12]={8,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[34][12]={4,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[35][12]={1,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][12]={1,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][12]={0,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][12]={2,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][12]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][12]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][12]={0,5,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[115][12]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][12]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][12]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][12]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[126][12]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][12]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][12]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][12]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][12]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][12]={0,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][12]={1,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[135][12]={2,-1,36,{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][12]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][12]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[138][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[148][12]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[149][12]={1,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][12]={1,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][12]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][12]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][12]={4,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][12]={4,3,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][12]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][12]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][12]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][12]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][13]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][13]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][13]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][13]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][13]={0,5,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][13]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][13]={1,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][13]={0,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][13]={1,3,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[39][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][13]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][13]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][13]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][13]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][13]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][13]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][13]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[119][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][13]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][13]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[126][13]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][13]={4,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{1,0,0}}
 MapToConvert[129][13]={4,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[130][13]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[131][13]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[132][13]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[133][13]={1,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[134][13]={1,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[135][13]={1,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][13]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][13]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[138][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][13]={1,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][13]={0,5,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][13]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][13]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[204][13]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][13]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][13]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][13]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][13]={4,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][13]={3,3,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][13]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[211][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][13]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][13]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][14]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][14]={6,4,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][14]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][14]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][14]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][14]={3,3,9,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[36][14]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][14]={5,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][14]={1,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][14]={1,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[40][14]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][14]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][14]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][14]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][14]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][14]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][14]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][14]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[121][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][14]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][14]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][14]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][14]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][14]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][14]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][14]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][14]={7,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[131][14]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][14]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][14]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][14]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][14]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][14]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][14]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][14]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][14]={0,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][14]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[153][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[205][14]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][14]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][14]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][14]={4,3,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][14]={4,3,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][14]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][14]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][14]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][15]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][15]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][15]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][15]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][15]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][15]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][15]={3,-1,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[36][15]={3,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][15]={3,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][15]={4,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][15]={4,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][15]={4,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[113][15]={4,3,36,{{0,-1},{0,-1},{1,4}},{-1,1},{1,0,0}}
 MapToConvert[114][15]={0,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[115][15]={4,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][15]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][15]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][15]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][15]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][15]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[121][15]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][15]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][15]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][15]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][15]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][15]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][15]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][15]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][15]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][15]={7,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[131][15]={7,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[132][15]={7,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[133][15]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][15]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][15]={1,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][15]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][15]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][15]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][15]={1,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][15]={1,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[205][15]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][15]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][15]={4,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][15]={4,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][15]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][15]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][15]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][15]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[213][15]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][15]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][15]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][16]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][16]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][16]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][16]={6,4,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][16]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][16]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][16]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][16]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][16]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][16]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][16]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][16]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[42][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[113][16]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[114][16]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][16]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][16]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][16]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][16]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][16]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][16]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][16]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][16]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][16]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][16]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][16]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][16]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][16]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][16]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][16]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][16]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][16]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][16]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][16]={3,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[134][16]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][16]={1,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][16]={1,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][16]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][16]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][16]={0,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[149][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][16]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][16]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][16]={4,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][16]={4,3,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][16]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][16]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][16]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][16]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][16]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][16]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][16]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][17]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][17]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][17]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][17]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][17]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][17]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][17]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][17]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][17]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][17]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][17]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][17]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][17]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][17]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][17]={0,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][17]={1,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[48][17]={1,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[49][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][17]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][17]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][17]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][17]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][17]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][17]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][17]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][17]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][17]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][17]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][17]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][17]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][17]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][17]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][17]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][17]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][17]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][17]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][17]={6,0,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][17]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][17]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][17]={6,0,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[134][17]={1,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[135][17]={1,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[136][17]={1,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][17]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][17]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[147][17]={0,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[149][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[204][17]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][17]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][17]={4,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][17]={4,3,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][17]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][17]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][17]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][17]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][17]={4,3,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][17]={3,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][17]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][17]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[28][18]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[29][18]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][18]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][18]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][18]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][18]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][18]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][18]={4,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][18]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][18]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][18]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][18]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][18]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][18]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][18]={0,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][18]={0,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][18]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][18]={1,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][18]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][18]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][18]={6,0,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[131][18]={6,0,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][18]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][18]={3,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[135][18]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][18]={1,3,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[137][18]={2,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][18]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[205][18]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][18]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][18]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][18]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][18]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][18]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][18]={3,3,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][18]={3,3,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[213][18]={3,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][18]={0,5,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][18]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][18]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][19]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[28][19]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][19]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][19]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][19]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][19]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][19]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][19]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][19]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][19]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][19]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][19]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][19]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][19]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][19]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[46][19]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][19]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][19]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][19]={1,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[111][19]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][19]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][19]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][19]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][19]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][19]={7,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[131][19]={6,0,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][19]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][19]={3,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[135][19]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][19]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][19]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][19]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[139][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][19]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][19]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][19]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][19]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][19]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][19]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][19]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][19]={3,3,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][19]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][19]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][19]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][19]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[216][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][19]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][19]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][20]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][20]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][20]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][20]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][20]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][20]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][20]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][20]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][20]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][20]={4,2,9,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[37][20]={3,2,9,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[38][20]={3,2,9,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[39][20]={3,-1,9,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[40][20]={3,3,9,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][20]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[42][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[43][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][20]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][20]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][20]={4,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][20]={1,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][20]={0,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[63][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][20]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][20]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][20]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][20]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][20]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][20]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][20]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][20]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][20]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][20]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][20]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][20]={0,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[136][20]={0,5,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][20]={1,2,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[138][20]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[139][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][20]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][20]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][20]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][20]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][20]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][20]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][20]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][20]={1,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][20]={0,5,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][20]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][20]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][20]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[217][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][20]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][20]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[26][21]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][21]={6,4,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][21]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][21]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][21]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][21]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][21]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][21]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][21]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][21]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][21]={4,2,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][21]={3,2,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][21]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][21]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][21]={3,-1,9,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][21]={3,3,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][21]={3,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[48][21]={3,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[49][21]={0,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][21]={1,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[51][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][21]={0,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][21]={0,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][21]={3,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[112][21]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[113][21]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[114][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][21]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][21]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][21]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][21]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][21]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][21]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][21]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][21]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][21]={0,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][21]={1,2,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[138][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[139][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[178][21]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][21]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[180][21]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][21]={6,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][21]={7,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][21]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][21]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][21]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][21]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][21]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][21]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][21]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][21]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][21]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][21]={0,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][21]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][21]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][21]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][22]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[27][22]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][22]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][22]={6,4,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][22]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][22]={4,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][22]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][22]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][22]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][22]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][22]={3,3,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][22]={4,3,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][22]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][22]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][22]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][22]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][22]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][22]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][22]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][22]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][22]={0,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[52][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[112][22]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][22]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][22]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][22]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][22]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][22]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][22]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][22]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][22]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][22]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][22]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][22]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][22]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][22]={0,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[179][22]={0,-1,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][22]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][22]={6,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][22]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][22]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][22]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][22]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][22]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][22]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][22]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][22]={15,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][22]={0,3,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][22]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][22]={0,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][22]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[219][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][22]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][22]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][23]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][23]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][23]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][23]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][23]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][23]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][23]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][23]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][23]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][23]={3,2,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][23]={3,-1,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][23]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][23]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][23]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][23]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][23]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][23]={3,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][23]={3,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][23]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][23]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][23]={0,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[112][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][23]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][23]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][23]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][23]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][23]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][23]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][23]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][23]={3,2,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][23]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][23]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[144][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][23]={3,-1,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[205][23]={6,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][23]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][23]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][23]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][23]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][23]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][23]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][23]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][23]={0,-1,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][23]={0,-1,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][23]={0,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][23]={0,3,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][23]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][23]={1,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[219][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][23]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][23]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][24]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][24]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][24]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][24]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][24]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][24]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][24]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][24]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][24]={3,-1,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][24]={3,3,9,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[36][24]={3,-1,9,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][24]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][24]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][24]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][24]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][24]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][24]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[47][24]={3,-1,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[48][24]={3,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][24]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][24]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][24]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][24]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][24]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][24]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][24]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][24]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][24]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][24]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][24]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][24]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][24]={3,2,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[136][24]={3,2,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[137][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[138][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][24]={0,5,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[144][24]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[145][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][24]={6,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][24]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][24]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][24]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][24]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][24]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][24]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][24]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][24]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][24]={0,3,10,{{0,-1},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[215][24]={3,3,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[216][24]={0,-1,10,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[217][24]={1,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][24]={1,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][24]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][24]={1,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[221][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][24]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][24]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[26][25]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][25]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][25]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][25]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][25]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][25]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][25]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][25]={4,-1,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][25]={3,3,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][25]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][25]={3,3,9,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][25]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][25]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][25]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][25]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][25]={4,3,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][25]={3,-1,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][25]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[44][25]={4,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[45][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[47][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[48][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][25]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][25]={4,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][25]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][25]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[115][25]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][25]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][25]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][25]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][25]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][25]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][25]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][25]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][25]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][25]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][25]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][25]={3,2,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][25]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][25]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][25]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][25]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][25]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][25]={3,2,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][25]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][25]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][25]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[136][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][25]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][25]={6,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][25]={7,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][25]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][25]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][25]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][25]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][25]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][25]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][25]={1,3,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][25]={0,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][25]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][25]={3,-1,10,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[217][25]={4,3,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[218][25]={0,-1,10,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[219][25]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][25]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[221][25]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[222][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][25]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][25]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][26]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][26]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][26]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][26]={6,4,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][26]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][26]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][26]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][26]={4,-1,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][26]={3,-1,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][26]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][26]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][26]={3,3,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[38][26]={3,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[39][26]={4,-1,9,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[40][26]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][26]={4,-1,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][26]={3,-1,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][26]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][26]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][26]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][26]={4,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][26]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][26]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][26]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][26]={3,2,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][26]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][26]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][26]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][26]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][26]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][26]={3,2,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[126][26]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][26]={0,2,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[128][26]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][26]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][26]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][26]={3,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][26]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[133][26]={3,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[136][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[137][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][26]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][26]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][26]={6,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][26]={8,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][26]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][26]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][26]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][26]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][26]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][26]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][26]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][26]={0,-1,10,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[215][26]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][26]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][26]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][26]={0,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][26]={2,-1,10,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[220][26]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][26]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][26]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][26]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][26]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[26][27]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][27]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][27]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][27]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][27]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][27]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][27]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][27]={4,3,9,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[34][27]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][27]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][27]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][27]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][27]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][27]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][27]={4,-1,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][27]={4,-1,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][27]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][27]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][27]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][27]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[52][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][27]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][27]={3,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][27]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][27]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][27]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][27]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][27]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][27]={3,2,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[126][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[129][27]={3,2,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][27]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[131][27]={4,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][27]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][27]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[204][27]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[205][27]={6,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][27]={8,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][27]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][27]={15,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[209][27]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][27]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][27]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][27]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][27]={1,3,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][27]={0,3,10,{{1,2},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[215][27]={0,-1,10,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[216][27]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][27]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][27]={0,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][27]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][27]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][27]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][27]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][27]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][27]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][27]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][28]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][28]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][28]={6,4,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][28]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][28]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][28]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][28]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][28]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][28]={4,2,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[35][28]={3,2,9,{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][28]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][28]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][28]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][28]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][28]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][28]={4,-1,9,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[42][28]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][28]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][28]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[52][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[53][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[118][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[119][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[121][28]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][28]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][28]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][28]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][28]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[126][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[127][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][28]={3,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][28]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][28]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[134][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][28]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][28]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][28]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[164][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[203][28]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[204][28]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][28]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][28]={8,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][28]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][28]={0,5,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][28]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[210][28]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][28]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][28]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][28]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][28]={3,2,10,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[215][28]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][28]={3,2,10,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[217][28]={3,2,10,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[218][28]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][28]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][28]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][28]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][28]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][28]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][28]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][28]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][28]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][29]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][29]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][29]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][29]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][29]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][29]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][29]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][29]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][29]={4,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][29]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][29]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][29]={3,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][29]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][29]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][29]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][29]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][29]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][29]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][29]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][29]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[105][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][29]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][29]={3,2,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][29]={3,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][29]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][29]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][29]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[126][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][29]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][29]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[132][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[133][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][29]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[155][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][29]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][29]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[164][29]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[165][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][29]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][29]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][29]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][29]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][29]={0,5,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][29]={15,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[209][29]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][29]={10,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][29]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][29]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][29]={3,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][29]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][29]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][29]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][29]={4,2,10,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[218][29]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][29]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][29]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][29]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][29]={0,5,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][29]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][29]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][29]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][29]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][29]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[26][30]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[27][30]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][30]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][30]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][30]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][30]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][30]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][30]={1,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][30]={4,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][30]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][30]={0,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][30]={3,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][30]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][30]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][30]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][30]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][30]={15,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][30]={4,2,9,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[44][30]={4,2,9,{{1,2},{0,-1},{1,1}},{-1,1},{0,1,0}}
 MapToConvert[45][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[46][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][30]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][30]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][30]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[113][30]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][30]={4,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][30]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][30]={4,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][30]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][30]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][30]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][30]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[127][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[130][30]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[131][30]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[145][30]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[146][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][30]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][30]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][30]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][30]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][30]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][30]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][30]={15,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[209][30]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[210][30]={10,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][30]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][30]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][30]={3,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][30]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][30]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][30]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][30]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][30]={0,-1,10,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[219][30]={3,2,10,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[220][30]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][30]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][30]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][30]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][30]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][30]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[226][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][30]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][30]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][31]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][31]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][31]={3,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][31]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][31]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][31]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][31]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][31]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][31]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][31]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][31]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][31]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][31]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][31]={3,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][31]={4,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[40][31]={4,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][31]={15,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[42][31]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[43][31]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][31]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][31]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[107][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][31]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][31]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][31]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[112][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][31]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[119][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[122][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[123][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[124][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[127][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][31]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][31]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][31]={3,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][31]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][31]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][31]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][31]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][31]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][31]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][31]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][31]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][31]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[209][31]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][31]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][31]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][31]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][31]={3,2,10,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[214][31]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][31]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][31]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][31]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][31]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][31]={3,2,10,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[220][31]={3,-1,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[221][31]={3,2,10,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[222][31]={4,2,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[223][31]={5,-1,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[224][31]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][31]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[226][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][31]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][31]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][32]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][32]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][32]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][32]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][32]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][32]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][32]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][32]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][32]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][32]={3,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[35][32]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[36][32]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][32]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][32]={3,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][32]={4,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][32]={2,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][32]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][32]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][32]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][32]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[45][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][32]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[101][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][32]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][32]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][32]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][32]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][32]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][32]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[132][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][32]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[202][32]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[203][32]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][32]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][32]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][32]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][32]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][32]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][32]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][32]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][32]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][32]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][32]={3,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][32]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][32]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][32]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][32]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][32]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][32]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][32]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][32]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][32]={1,-1,10,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[223][32]={4,2,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[224][32]={5,-1,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][32]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][32]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][32]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][32]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][33]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][33]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[26][33]={3,2,9,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[27][33]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[28][33]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[29][33]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][33]={4,2,9,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[31][33]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][33]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][33]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][33]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][33]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][33]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][33]={3,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][33]={4,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[39][33]={15,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][33]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][33]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][33]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][33]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][33]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[45][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][33]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][33]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[101][33]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[102][33]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][33]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][33]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[105][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][33]={3,2,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][33]={3,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[109][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][33]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][33]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][33]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][33]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][33]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][33]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][33]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][33]={10,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][33]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][33]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][33]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][33]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][33]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][33]={3,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][33]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][33]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][33]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][33]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][33]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][33]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][33]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][33]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][33]={4,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][33]={4,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][33]={4,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][33]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][33]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][33]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][33]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][34]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[25][34]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[26][34]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][34]={3,2,9,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[28][34]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][34]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][34]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][34]={3,2,9,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[32][34]={3,-1,9,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[33][34]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][34]={0,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][34]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][34]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][34]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][34]={4,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][34]={15,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[40][34]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[41][34]={2,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][34]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][34]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][34]={0,5,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][34]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[46][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[63][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[64][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][34]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][34]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[102][34]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][34]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][34]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[108][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][34]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[111][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][34]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[121][34]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][34]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][34]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[201][34]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][34]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][34]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][34]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][34]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][34]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][34]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][34]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][34]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][34]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][34]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][34]={3,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][34]={3,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][34]={4,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[215][34]={4,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][34]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][34]={4,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][34]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][34]={4,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][34]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][34]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][34]={4,2,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[223][34]={4,2,10,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[224][34]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][34]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][34]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][34]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][34]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][35]={0,5,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][35]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][35]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][35]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][35]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][35]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][35]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][35]={3,2,9,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[33][35]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[34][35]={3,2,9,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[35][35]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][35]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][35]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][35]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][35]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[40][35]={15,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][35]={15,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][35]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][35]={3,2,9,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[44][35]={3,-1,9,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[45][35]={3,2,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[46][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][35]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[101][35]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][35]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][35]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][35]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][35]={3,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][35]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[112][35]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][35]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][35]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][35]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][35]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][35]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][35]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[132][35]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[133][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][35]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][35]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[137][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][35]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[200][35]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[201][35]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][35]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][35]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][35]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][35]={5,-1,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[206][35]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][35]={4,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[208][35]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][35]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][35]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][35]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][35]={3,2,10,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[213][35]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][35]={4,2,10,{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][35]={4,-1,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][35]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][35]={4,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[218][35]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][35]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[220][35]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][35]={3,-1,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][35]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][35]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[224][35]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][35]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][35]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][35]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][35]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][35]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][36]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][36]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][36]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][36]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][36]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][36]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][36]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][36]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][36]={3,2,9,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[34][36]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][36]={3,2,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[36][36]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][36]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][36]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][36]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][36]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[41][36]={15,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][36]={15,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[43][36]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][36]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][36]={8,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][36]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[47][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][36]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][36]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][36]={2,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][36]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][36]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][36]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[106][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][36]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][36]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][36]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][36]={3,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[112][36]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][36]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[114][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][36]={4,2,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][36]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][36]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][36]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][36]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][36]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][36]={3,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][36]={4,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][36]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][36]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][36]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[199][36]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][36]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][36]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][36]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][36]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][36]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][36]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][36]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][36]={4,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][36]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][36]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][36]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][36]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][36]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][36]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][36]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[215][36]={4,-1,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][36]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][36]={4,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][36]={4,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][36]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][36]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][36]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][36]={4,-1,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[223][36]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[224][36]={3,2,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[225][36]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][36]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][36]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][36]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[229][36]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[0][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[25][37]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[26][37]={3,2,9,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[27][37]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][37]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][37]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][37]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][37]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][37]={3,2,9,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[33][37]={3,-1,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[34][37]={3,2,9,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[35][37]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[36][37]={3,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][37]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[38][37]={4,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[39][37]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][37]={4,2,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][37]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[42][37]={5,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[43][37]={1,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][37]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][37]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][37]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][37]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[79][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[95][37]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][37]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][37]={4,2,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][37]={3,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][37]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][37]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][37]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][37]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][37]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][37]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][37]={4,2,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[112][37]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][37]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][37]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][37]={4,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[118][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[119][37]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][37]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][37]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][37]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][37]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][37]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][37]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][37]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][37]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][37]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][37]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][37]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][37]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[199][37]={4,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][37]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][37]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][37]={5,-1,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][37]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][37]={5,-1,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][37]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][37]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][37]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][37]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][37]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][37]={4,2,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[211][37]={3,2,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[212][37]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][37]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][37]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][37]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][37]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][37]={4,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[218][37]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][37]={0,-1,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[220][37]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][37]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][37]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][37]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][37]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[225][37]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][37]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][37]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][37]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[229][37]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][38]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[26][38]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][38]={3,-1,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[28][38]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[29][38]={3,2,9,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[30][38]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][38]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][38]={4,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][38]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][38]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][38]={3,2,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[36][38]={3,2,9,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][38]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][38]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][38]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][38]={4,2,9,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][38]={4,-1,9,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][38]={2,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][38]={4,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][38]={2,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][38]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][38]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][38]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][38]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,0}}
 MapToConvert[49][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[96][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[98][38]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][38]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][38]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][38]={0,5,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[108][38]={3,2,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[109][38]={3,2,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[110][38]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][38]={4,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][38]={4,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][38]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][38]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][38]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[118][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][38]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][38]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][38]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[125][38]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][38]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][38]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][38]={4,2,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[129][38]={4,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[130][38]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][38]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][38]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[133][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[134][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][38]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][38]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][38]={3,2,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[152][38]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[153][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][38]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][38]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][38]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][38]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][38]={4,-1,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[202][38]={4,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][38]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][38]={4,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][38]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][38]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][38]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][38]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][38]={4,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][38]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][38]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][38]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[213][38]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][38]={3,-1,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[215][38]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][38]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][38]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][38]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[219][38]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][38]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[221][38]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][38]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][38]={3,-1,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[224][38]={3,-1,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[225][38]={3,2,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[226][38]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][38]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][38]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][38]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][39]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][39]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][39]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][39]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][39]={4,2,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[30][39]={4,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[31][39]={3,2,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[32][39]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][39]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][39]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][39]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][39]={3,-1,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][39]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][39]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][39]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][39]={4,2,9,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[41][39]={2,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][39]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][39]={5,-1,-1,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][39]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][39]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][39]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][39]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][39]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][39]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[50][39]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[51][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][39]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][39]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][39]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[85][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][39]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[97][39]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[98][39]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][39]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[103][39]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[104][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][39]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][39]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][39]={3,2,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[110][39]={3,2,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[111][39]={4,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[112][39]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][39]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][39]={2,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[117][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[118][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][39]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][39]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][39]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[125][39]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][39]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][39]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][39]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][39]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][39]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][39]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][39]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][39]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][39]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[198][39]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][39]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][39]={3,-1,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][39]={4,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][39]={4,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[203][39]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][39]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[205][39]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][39]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][39]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][39]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][39]={4,2,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[210][39]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[211][39]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[212][39]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[213][39]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[214][39]={3,2,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[215][39]={3,-1,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[216][39]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][39]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][39]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[219][39]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][39]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[221][39]={0,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][39]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][39]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][39]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][39]={3,-1,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[226][39]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][39]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][40]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[11][40]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[12][40]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][40]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][40]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][40]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][40]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][40]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][40]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][40]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][40]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][40]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][40]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][40]={0,5,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][40]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][40]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][40]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][40]={1,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][40]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][40]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][40]={4,2,9,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[42][40]={4,2,9,{{1,5},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[43][40]={3,2,9,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[44][40]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][40]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][40]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][40]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][40]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][40]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][40]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][40]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][40]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][40]={4,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][40]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][40]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[86][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][40]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][40]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][40]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][40]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][40]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[104][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[109][40]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][40]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][40]={4,2,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[112][40]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][40]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][40]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][40]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][40]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[119][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][40]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][40]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][40]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[126][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][40]={4,2,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[128][40]={4,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][40]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][40]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[133][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[134][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][40]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[139][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][40]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[199][40]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][40]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][40]={5,-1,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[202][40]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[203][40]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[204][40]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[205][40]={3,2,10,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[206][40]={3,2,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[207][40]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][40]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][40]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][40]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][40]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][40]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][40]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][40]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][40]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[216][40]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[217][40]={0,5,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[218][40]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][40]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[220][40]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][40]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][40]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][40]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][40]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[225][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][40]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][41]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][41]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][41]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][41]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][41]={3,2,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][41]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[15][41]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[16][41]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[17][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][41]={3,2,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][41]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][41]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][41]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][41]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][41]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][41]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][41]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][41]={4,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][41]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][41]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][41]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][41]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][41]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][41]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][41]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][41]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][41]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][41]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][41]={1,-1,9,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[43][41]={4,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][41]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][41]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][41]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][41]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][41]={8,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][41]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][41]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][41]={6,4,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][41]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][41]={4,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][41]={4,2,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][41]={4,2,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][41]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][41]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][41]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][41]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][41]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][41]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][41]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][41]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[104][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[109][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[110][41]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][41]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][41]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][41]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][41]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][41]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[126][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][41]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[128][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[129][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[131][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][41]={4,3,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[200][41]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][41]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][41]={0,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][41]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[204][41]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[205][41]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][41]={0,-1,10,{{0,-1},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[207][41]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[208][41]={3,-1,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[209][41]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[210][41]={3,2,10,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[211][41]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[212][41]={3,-1,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[213][41]={3,2,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[214][41]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][41]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[216][41]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[217][41]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[218][41]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][41]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[220][41]={3,-1,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[221][41]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][41]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[225][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][41]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][41]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][42]={3,2,34,{{0,-1},{1,3},{1,4}},{-1,1},{1,0,0}}
 MapToConvert[10][42]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][42]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][42]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][42]={3,2,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][42]={3,2,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][42]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][42]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][42]={4,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][42]={0,5,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][42]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][42]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][42]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[23][42]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[24][42]={3,2,34,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[25][42]={3,2,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][42]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][42]={4,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][42]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][42]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][42]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][42]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][42]={4,2,9,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[33][42]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][42]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][42]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][42]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][42]={4,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][42]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][42]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][42]={4,-1,9,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][42]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][42]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][42]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][42]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][42]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][42]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][42]={8,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][42]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][42]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][42]={8,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][42]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][42]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][42]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[54][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[80][42]={4,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[81][42]={4,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][42]={3,2,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][42]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][42]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][42]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][42]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][42]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][42]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][42]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][42]={4,2,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[103][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[111][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[112][42]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][42]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][42]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][42]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][42]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][42]={2,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][42]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[203][42]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][42]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][42]={3,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][42]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[207][42]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][42]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][42]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][42]={3,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][42]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][42]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][42]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][42]={3,2,10,{{0,-1},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[215][42]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[216][42]={3,2,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[217][42]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[218][42]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][42]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][42]={3,2,10,{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][42]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][42]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][42]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][42]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][43]={3,2,34,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[10][43]={4,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][43]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][43]={3,2,34,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[14][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][43]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][43]={4,2,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][43]={3,2,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][43]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][43]={3,2,34,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[25][43]={3,-1,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][43]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][43]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][43]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][43]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][43]={3,2,9,{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][43]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][43]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][43]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][43]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][43]={3,2,9,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[38][43]={3,-1,9,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[39][43]={4,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][43]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][43]={3,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][43]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][43]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][43]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][43]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][43]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][43]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][43]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][43]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][43]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][43]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][43]={7,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][43]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[54][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][43]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][43]={4,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][43]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][43]={4,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[83][43]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][43]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][43]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][43]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][43]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][43]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][43]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][43]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][43]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][43]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][43]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][43]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][43]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][43]={4,2,10,{{0,-1},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[201][43]={2,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][43]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][43]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][43]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[205][43]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][43]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][43]={3,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][43]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[209][43]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[210][43]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][43]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][43]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][43]={3,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][43]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][43]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][43]={3,2,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[217][43]={0,5,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[218][43]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][43]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[220][43]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][43]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][43]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][43]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[8][44]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[9][44]={4,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][44]={4,2,34,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[11][44]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][44]={3,2,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][44]={3,2,34,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[14][44]={3,2,34,{{1,2},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][44]={3,2,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][44]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][44]={3,2,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][44]={3,2,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][44]={4,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][44]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][44]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][44]={4,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][44]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][44]={3,2,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][44]={4,2,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][44]={4,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][44]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][44]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][44]={4,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][44]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][44]={3,-1,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][44]={3,2,34,{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][44]={3,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][44]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][44]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][44]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][44]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][44]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][44]={3,-1,9,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[40][44]={3,2,9,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][44]={3,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][44]={4,2,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][44]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][44]={5,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][44]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][44]={4,2,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][44]={4,2,9,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[48][44]={4,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][44]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[50][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[51][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[52][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[53][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[54][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[55][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][44]={3,2,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][44]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][44]={4,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][44]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][44]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[85][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][44]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][44]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][44]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[102][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[103][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[113][44]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[114][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][44]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][44]={3,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][44]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[144][44]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[145][44]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][44]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][44]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][44]={4,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[201][44]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][44]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][44]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][44]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][44]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][44]={3,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][44]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[208][44]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][44]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][44]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][44]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][44]={3,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][44]={3,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[214][44]={3,2,10,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[215][44]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][44]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][44]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][44]={3,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[219][44]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][44]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][44]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][45]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][45]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][45]={0,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][45]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][45]={4,-1,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][45]={4,2,34,{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][45]={3,2,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][45]={3,3,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[15][45]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][45]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][45]={3,2,34,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[18][45]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][45]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][45]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][45]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][45]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][45]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][45]={3,3,34,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[25][45]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][45]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][45]={5,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][45]={5,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][45]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][45]={3,-1,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][45]={3,3,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[32][45]={3,-1,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][45]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][45]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][45]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][45]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][45]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][45]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][45]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][45]={3,-1,9,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][45]={3,3,9,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[42][45]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][45]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][45]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][45]={3,3,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][45]={5,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][45]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][45]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][45]={0,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][45]={1,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][45]={1,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][45]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][45]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[83][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][45]={3,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[99][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[113][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[114][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][45]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][45]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][45]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][45]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][45]={0,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][45]={1,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[145][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[198][45]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][45]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][45]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][45]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][45]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][45]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][45]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][45]={4,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][45]={4,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][45]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][45]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][45]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][45]={2,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][45]={3,-1,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[212][45]={3,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][45]={0,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][45]={1,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][45]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][45]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][45]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][45]={0,3,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[219][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][45]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][45]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][46]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][46]={4,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][46]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][46]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][46]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][46]={4,-1,34,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[13][46]={3,2,34,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[14][46]={3,2,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][46]={3,2,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][46]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][46]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][46]={3,3,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][46]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][46]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][46]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][46]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][46]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][46]={3,3,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][46]={3,3,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][46]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][46]={5,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][46]={4,-1,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][46]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][46]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][46]={3,3,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[32][46]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][46]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][46]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][46]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][46]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][46]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][46]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][46]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][46]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][46]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][46]={3,3,9,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[43][46]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][46]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][46]={4,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][46]={5,-1,9,{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][46]={4,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][46]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[51][46]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[52][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][46]={1,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][46]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][46]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][46]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][46]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[84][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][46]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][46]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][46]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[104][46]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][46]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[120][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[121][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[146][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][46]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][46]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][46]={1,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][46]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][46]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][46]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][46]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][46]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][46]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][46]={4,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][46]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][46]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][46]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][46]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][46]={4,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][46]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][46]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][46]={3,2,10,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[215][46]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][46]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][46]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][46]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][46]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[220][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][46]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][46]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][47]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][47]={4,2,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][47]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][47]={4,3,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][47]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][47]={3,-1,34,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[12][47]={3,3,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][47]={3,2,34,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[14][47]={3,2,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][47]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][47]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][47]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][47]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][47]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][47]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][47]={3,-1,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][47]={4,-1,34,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[23][47]={3,3,34,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[24][47]={3,3,34,{{1,2},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[25][47]={3,3,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[26][47]={3,3,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[27][47]={4,3,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[28][47]={3,-1,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][47]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][47]={15,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][47]={3,-1,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][47]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][47]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][47]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][47]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][47]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][47]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][47]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][47]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][47]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][47]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][47]={3,-1,9,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[43][47]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][47]={3,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][47]={3,3,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][47]={3,-1,9,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[47][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][47]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][47]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][47]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][47]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][47]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[54][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[77][47]={0,2,33,{{0,-1},{0,-1},{1,4}},{-1,1},{1,0,0}}
 MapToConvert[78][47]={1,2,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[79][47]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][47]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][47]={4,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[82][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][47]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][47]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][47]={0,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][47]={0,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][47]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][47]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][47]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[119][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][47]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[122][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][47]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][47]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[199][47]={1,-1,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][47]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][47]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][47]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][47]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][47]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][47]={5,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][47]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][47]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][47]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][47]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][47]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][47]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][47]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][47]={2,-1,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][47]={3,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][47]={0,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][47]={0,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][47]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][47]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[220][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][47]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][47]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[6][48]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][48]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][48]={3,2,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[9][48]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][48]={3,-1,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[11][48]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][48]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][48]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][48]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][48]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][48]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][48]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][48]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][48]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][48]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][48]={3,3,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][48]={3,-1,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][48]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][48]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][48]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][48]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][48]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][48]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][48]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][48]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][48]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][48]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][48]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][48]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][48]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][48]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][48]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][48]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][48]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][48]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][48]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][48]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][48]={3,3,9,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[44][48]={3,-1,9,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[45][48]={6,0,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][48]={3,3,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][48]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][48]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][48]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][48]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][48]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][48]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[56][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][48]={0,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][48]={0,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][48]={1,3,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[80][48]={4,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][48]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][48]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[92][48]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[93][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][48]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[99][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][48]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[101][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][48]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][48]={3,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][48]={0,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][48]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][48]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][48]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[123][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][48]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][48]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[200][48]={1,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[201][48]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][48]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][48]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][48]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][48]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][48]={4,2,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[207][48]={3,2,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[208][48]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][48]={3,2,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][48]={3,-1,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[211][48]={0,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][48]={0,5,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][48]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][48]={2,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][48]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][48]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][48]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[219][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][48]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][48]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][49]={1,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[1][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[6][49]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][49]={3,2,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][49]={3,-1,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[9][49]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][49]={7,-1,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[11][49]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][49]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][49]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][49]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][49]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][49]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][49]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][49]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][49]={3,3,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][49]={3,-1,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][49]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][49]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][49]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][49]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][49]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][49]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][49]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][49]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][49]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][49]={3,3,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][49]={3,3,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][49]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][49]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[46][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[49][49]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][49]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][49]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][49]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][49]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][49]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][49]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][49]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][49]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][49]={0,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][49]={0,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][49]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][49]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][49]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][49]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[97][49]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][49]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][49]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][49]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][49]={3,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][49]={3,2,36,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[104][49]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][49]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[106][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][49]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][49]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][49]={1,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[201][49]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][49]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][49]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][49]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][49]={3,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][49]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][49]={3,2,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[208][49]={3,-1,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[209][49]={3,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][49]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][49]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][49]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][49]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][49]={0,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[215][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][49]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][49]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][50]={1,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][50]={3,3,34,{{1,5},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[8][50]={3,3,34,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[9][50]={6,0,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][50]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][50]={3,-1,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[12][50]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][50]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][50]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][50]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][50]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][50]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][50]={3,3,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][50]={6,0,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][50]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][50]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][50]={3,-1,9,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][50]={3,-1,9,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][50]={3,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][50]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[46][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[47][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][50]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[50][50]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][50]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][50]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][50]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][50]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][50]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[58][50]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[59][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][50]={0,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][50]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][50]={4,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][50]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][50]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][50]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[94][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[98][50]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][50]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][50]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][50]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][50]={3,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][50]={3,2,36,{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][50]={3,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][50]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][50]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[107][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][50]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][50]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][50]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][50]={4,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[202][50]={5,-1,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][50]={4,-1,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[204][50]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][50]={3,-1,10,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][50]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][50]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][50]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][50]={4,2,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[210][50]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][50]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][50]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][50]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][50]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[216][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][50]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][50]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][51]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][51]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][51]={6,0,34,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[12][51]={3,3,34,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[13][51]={6,0,34,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[14][51]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][51]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][51]={7,-1,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][51]={6,0,34,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[20][51]={6,0,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][51]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][51]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][51]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][51]={3,3,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[43][51]={6,0,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][51]={6,-1,9,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[45][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][51]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][51]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][51]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][51]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][51]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][51]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][51]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][51]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][51]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][51]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][51]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][51]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][51]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[76][51]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[77][51]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][51]={4,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][51]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][51]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][51]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][51]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[94][51]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[95][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][51]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][51]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][51]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][51]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][51]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][51]={3,3,36,{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][51]={4,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][51]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][51]={4,2,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[106][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][51]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][51]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][51]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[200][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][51]={4,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][51]={4,3,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[203][51]={4,3,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][51]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][51]={4,-1,10,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[206][51]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][51]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][51]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][51]={1,2,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[210][51]={1,2,10,{{1,2},{0,-1},{1,1}},{-1,1},{0,1,0}}
 MapToConvert[211][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][51]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][51]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][51]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][52]={6,0,34,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[15][52]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][52]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][52]={7,-1,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][52]={6,0,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][52]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][52]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][52]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][52]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][52]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][52]={3,3,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][52]={6,0,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][52]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][52]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][52]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][52]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][52]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][52]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][52]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][52]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][52]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][52]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][52]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][52]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][52]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[75][52]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][52]={1,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][52]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][52]={3,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][52]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][52]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][52]={3,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][52]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][52]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][52]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][52]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][52]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][52]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][52]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][52]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][52]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][52]={3,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][52]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][52]={4,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[105][52]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][52]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][52]={3,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][52]={4,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][52]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[197][52]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][52]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[199][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[202][52]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[204][52]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[207][52]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][52]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][52]={1,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[210][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[212][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][52]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][52]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][53]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][53]={6,0,34,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[15][53]={6,0,34,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[16][53]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][53]={6,0,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][53]={6,0,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][53]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][53]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][53]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][53]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][53]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][53]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][53]={6,0,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][53]={6,0,34,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[42][53]={6,0,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[43][53]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][53]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][53]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][53]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][53]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][53]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][53]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][53]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][53]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][53]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][53]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][53]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][53]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][53]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][53]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[62][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[74][53]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[75][53]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][53]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][53]={4,2,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][53]={4,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][53]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[81][53]={4,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[82][53]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][53]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[84][53]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[93][53]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][53]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][53]={1,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][53]={4,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][53]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][53]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][53]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][53]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][53]={3,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][53]={3,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][53]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[104][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[105][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][53]={1,2,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][53]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][53]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][53]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[195][53]={0,3,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][53]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[197][53]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][53]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[199][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[204][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[207][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][53]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][53]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[6][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][54]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][54]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][54]={6,0,34,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[17][54]={6,0,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[18][54]={6,0,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][54]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][54]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][54]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][54]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][54]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][54]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][54]={6,0,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[42][54]={6,0,34,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[43][54]={6,0,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][54]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][54]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][54]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][54]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][54]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][54]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][54]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][54]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][54]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][54]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][54]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][54]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][54]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][54]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[75][54]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][54]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][54]={4,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[78][54]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][54]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[80][54]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][54]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][54]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][54]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][54]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][54]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[94][54]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][54]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][54]={1,2,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[97][54]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][54]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][54]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][54]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][54]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][54]={3,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][54]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[105][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[116][54]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][54]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][54]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[133][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][54]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][54]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[196][54]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][54]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][54]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][54]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][54]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[216][54]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][54]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][55]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[6][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][55]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][55]={6,0,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[42][55]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[46][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][55]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][55]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][55]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][55]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][55]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][55]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][55]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][55]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[59][55]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][55]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][55]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][55]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][55]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][55]={4,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[78][55]={4,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[79][55]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][55]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][55]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][55]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][55]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][55]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][55]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][55]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][55]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[88][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][55]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][55]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][55]={0,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][55]={0,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][55]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][55]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][55]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][55]={4,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[101][55]={4,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][55]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][55]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[106][55]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[107][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[116][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[193][55]={0,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][55]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][55]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][55]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][55]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][55]={0,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][55]={3,-1,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][55]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][55]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][56]={0,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[4][56]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[6][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][56]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][56]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][56]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][56]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][56]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][56]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][56]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][56]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][56]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][56]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][56]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][56]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][56]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][56]={6,0,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[43][56]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][56]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[47][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][56]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][56]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][56]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][56]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][56]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][56]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[55][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[56][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[57][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[60][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][56]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][56]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][56]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][56]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][56]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][56]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][56]={4,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[80][56]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][56]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][56]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][56]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][56]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][56]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][56]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][56]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][56]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[89][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][56]={4,2,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[94][56]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][56]={4,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][56]={4,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][56]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][56]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][56]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][56]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][56]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][56]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][56]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][56]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][56]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][56]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[191][56]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[192][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][56]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][56]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][56]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][56]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][56]={1,2,10,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[198][56]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][56]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][56]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[4][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[6][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][57]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][57]={6,0,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][57]={6,0,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[42][57]={6,0,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][57]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][57]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][57]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][57]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][57]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][57]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][57]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][57]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][57]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][57]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[62][57]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[63][57]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[64][57]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][57]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][57]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[67][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][57]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][57]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][57]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][57]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][57]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][57]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][57]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][57]={4,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[80][57]={3,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[81][57]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][57]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][57]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][57]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][57]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][57]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][57]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][57]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[89][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][57]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][57]={1,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[94][57]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][57]={0,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][57]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][57]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][57]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][57]={4,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[100][57]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][57]={4,2,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[102][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][57]={3,2,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][57]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][57]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[186][57]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[187][57]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[188][57]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[189][57]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][57]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][57]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][57]={2,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][57]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][57]={1,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][57]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][57]={3,2,10,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[198][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][57]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[207][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][57]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[212][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][57]={0,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][57]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][57]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][58]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][58]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][58]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][58]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][58]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][58]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][58]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][58]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][58]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][58]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][58]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][58]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][58]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][58]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][58]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][58]={6,0,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][58]={6,0,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][58]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][58]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][58]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][58]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][58]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][58]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][58]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][58]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][58]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][58]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][58]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][58]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][58]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][58]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][58]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][58]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][58]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][58]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][58]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][58]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[69][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][58]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][58]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][58]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][58]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][58]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][58]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][58]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][58]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][58]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][58]={4,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[82][58]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][58]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][58]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][58]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][58]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][58]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][58]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][58]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[90][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[93][58]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][58]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][58]={4,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[96][58]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][58]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][58]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][58]={3,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][58]={4,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][58]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][58]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[103][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][58]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[185][58]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[186][58]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[187][58]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][58]={4,2,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[189][58]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][58]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][58]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][58]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][58]={4,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[194][58]={1,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[198][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[203][58]={3,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][58]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][58]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][58]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][58]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][58]={1,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][58]={2,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][58]={4,2,10,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][58]={4,-1,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[213][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][58]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][58]={1,2,10,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[216][58]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][58]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[6][59]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][59]={6,0,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][59]={6,0,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[9][59]={8,-1,34,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[10][59]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][59]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][59]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][59]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][59]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][59]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][59]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][59]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][59]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][59]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][59]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][59]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][59]={6,0,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][59]={6,0,34,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][59]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[43][59]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[44][59]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][59]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][59]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][59]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][59]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][59]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][59]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[54][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[55][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][59]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][59]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][59]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][59]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][59]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][59]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][59]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][59]={3,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[65][59]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][59]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][59]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][59]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][59]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][59]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[71][59]={6,0,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[72][59]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][59]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][59]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][59]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][59]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][59]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][59]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][59]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][59]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][59]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][59]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][59]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][59]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][59]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][59]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][59]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][59]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][59]={0,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][59]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][59]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][59]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][59]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][59]={2,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][59]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][59]={2,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][59]={4,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[99][59]={4,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][59]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][59]={1,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[102][59]={3,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][59]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][59]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[115][59]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[116][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][59]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[158][59]={1,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][59]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][59]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][59]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[185][59]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][59]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][59]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][59]={4,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[189][59]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][59]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][59]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][59]={3,2,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][59]={4,2,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][59]={1,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][59]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][59]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[202][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[203][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][59]={3,2,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[205][59]={3,2,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[206][59]={4,-1,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[207][59]={1,2,10,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[208][59]={1,2,10,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][59]={0,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][59]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][59]={4,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[213][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][59]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][59]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][60]={3,-1,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[8][60]={3,3,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][60]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][60]={8,-1,34,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][60]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][60]={8,-1,34,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[13][60]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][60]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][60]={6,4,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][60]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][60]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][60]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][60]={4,3,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[29][60]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][60]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][60]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][60]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[33][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][60]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][60]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][60]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][60]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][60]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][60]={0,5,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][60]={0,-1,34,{{1,2},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][60]={0,-1,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][60]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][60]={6,0,34,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][60]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][60]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][60]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][60]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][60]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][60]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][60]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[57][60]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[58][60]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][60]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][60]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][60]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][60]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][60]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][60]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][60]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][60]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][60]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][60]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][60]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][60]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][60]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][60]={6,0,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[73][60]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][60]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][60]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][60]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][60]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][60]={0,5,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][60]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][60]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][60]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][60]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][60]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][60]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][60]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][60]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][60]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][60]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][60]={0,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][60]={0,5,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][60]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][60]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][60]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][60]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][60]={2,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][60]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][60]={2,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][60]={1,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][60]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][60]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][60]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][60]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][60]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][60]={1,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][60]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[106][60]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][60]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][60]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][60]={3,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][60]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[111][60]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][60]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][60]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][60]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][60]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][60]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[146][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[147][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[158][60]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[159][60]={5,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[160][60]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][60]={0,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][60]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][60]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][60]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][60]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][60]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][60]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][60]={0,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][60]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[191][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[192][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][60]={0,2,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][60]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][60]={0,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[196][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][60]={4,2,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][60]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][60]={3,-1,10,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[209][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[210][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][60]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][60]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[7][61]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][61]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][61]={4,-1,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][61]={4,-1,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[11][61]={3,-1,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][61]={3,-1,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[13][61]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][61]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][61]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][61]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][61]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][61]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][61]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][61]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][61]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][61]={3,-1,34,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[29][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[30][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[32][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[33][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][61]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][61]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][61]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][61]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][61]={0,5,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][61]={0,5,34,{{1,2},{1,0},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[40][61]={0,-1,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][61]={0,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][61]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][61]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][61]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][61]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][61]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][61]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][61]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][61]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][61]={6,0,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][61]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][61]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][61]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][61]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][61]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][61]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][61]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][61]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][61]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][61]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][61]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][61]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][61]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][61]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][61]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][61]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][61]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][61]={6,0,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[73][61]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][61]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][61]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][61]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][61]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][61]={0,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[79][61]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][61]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][61]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][61]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][61]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][61]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][61]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][61]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][61]={0,5,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][61]={0,2,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[89][61]={0,2,36,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[90][61]={1,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][61]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][61]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][61]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][61]={4,2,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[95][61]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][61]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][61]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][61]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][61]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][61]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][61]={0,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[102][61]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][61]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][61]={3,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][61]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][61]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][61]={1,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][61]={1,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[109][61]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][61]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][61]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][61]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][61]={0,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][61]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][61]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][61]={0,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][61]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][61]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][61]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][61]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][61]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][61]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][61]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][61]={0,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][61]={3,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][61]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][61]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][61]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[203][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[205][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][61]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][61]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][62]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[8][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[9][62]={3,-1,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[10][62]={4,3,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][62]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][62]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][62]={4,3,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[14][62]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][62]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][62]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][62]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][62]={8,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][62]={6,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][62]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][62]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][62]={6,0,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][62]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][62]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[27][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[28][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[34][62]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][62]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][62]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][62]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[38][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][62]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][62]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][62]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][62]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][62]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][62]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][62]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][62]={6,0,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][62]={6,0,33,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[55][62]={6,0,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][62]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][62]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][62]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][62]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][62]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][62]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][62]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][62]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][62]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][62]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][62]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][62]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][62]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][62]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][62]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][62]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][62]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][62]={6,0,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[74][62]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][62]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][62]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][62]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][62]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][62]={1,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[80][62]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][62]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][62]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][62]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][62]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][62]={0,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[86][62]={0,5,36,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[87][62]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[88][62]={0,2,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][62]={0,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][62]={0,2,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[91][62]={3,2,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[92][62]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][62]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][62]={4,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][62]={2,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][62]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][62]={4,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[98][62]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][62]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][62]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][62]={4,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[102][62]={0,5,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[103][62]={4,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][62]={3,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][62]={4,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[106][62]={4,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[107][62]={1,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[108][62]={4,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][62]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][62]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][62]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][62]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][62]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[114][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][62]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][62]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][62]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[121][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][62]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][62]={1,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[157][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][62]={1,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][62]={0,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[160][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[161][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[181][62]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[182][62]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][62]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][62]={4,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[185][62]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][62]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[187][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][62]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][62]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][62]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][62]={1,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][62]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][62]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][62]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][63]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[10][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[11][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][63]={4,3,34,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[14][63]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][63]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][63]={3,3,34,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][63]={3,-1,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[18][63]={5,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][63]={7,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][63]={8,-1,34,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[21][63]={7,-1,34,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[22][63]={3,-1,34,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[23][63]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[34][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[38][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][63]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][63]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][63]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][63]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][63]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][63]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][63]={6,0,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][63]={6,0,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][63]={6,0,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[55][63]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][63]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][63]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][63]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][63]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][63]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][63]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][63]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][63]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][63]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][63]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][63]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][63]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][63]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][63]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][63]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][63]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][63]={6,0,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][63]={6,0,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][63]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][63]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][63]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][63]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][63]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][63]={1,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[80][63]={0,-1,36,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[81][63]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[82][63]={0,-1,36,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[83][63]={0,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[84][63]={0,5,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][63]={3,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][63]={0,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][63]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][63]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][63]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][63]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][63]={3,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[92][63]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][63]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][63]={4,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[95][63]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][63]={2,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][63]={1,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][63]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][63]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][63]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][63]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][63]={0,3,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[103][63]={1,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[104][63]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][63]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][63]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][63]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][63]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][63]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][63]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][63]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][63]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][63]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][63]={1,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[156][63]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[180][63]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[181][63]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][63]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][63]={5,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[184][63]={4,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[185][63]={4,3,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[186][63]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[187][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[194][63]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][63]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[199][63]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][63]={0,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[201][63]={0,2,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[202][63]={0,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][63]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[206][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][63]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][63]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[16][64]={4,3,34,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[17][64]={4,-1,34,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][64]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][64]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][64]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][64]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][64]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][64]={3,-1,34,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[24][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[25][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][64]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[29][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][64]={3,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[41][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][64]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][64]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][64]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][64]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][64]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][64]={6,0,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][64]={6,0,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][64]={6,0,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][64]={6,0,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][64]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][64]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][64]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][64]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][64]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][64]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][64]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][64]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][64]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][64]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][64]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][64]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][64]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][64]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][64]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][64]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][64]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][64]={6,0,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[74][64]={6,0,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][64]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][64]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][64]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][64]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][64]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][64]={0,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][64]={0,5,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][64]={0,5,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][64]={0,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][64]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][64]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][64]={3,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[87][64]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][64]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][64]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][64]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][64]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][64]={0,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[93][64]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][64]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][64]={10,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[96][64]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][64]={1,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[98][64]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][64]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][64]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][64]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][64]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][64]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][64]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][64]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][64]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][64]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][64]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][64]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][64]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][64]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][64]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][64]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][64]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][64]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[121][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][64]={4,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][64]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][64]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][64]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][64]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][64]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][64]={3,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[187][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[195][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][64]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][64]={3,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[199][64]={0,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[202][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[203][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][64]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][64]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][65]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[10][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[16][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][65]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][65]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][65]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][65]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][65]={4,3,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][65]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][65]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[26][65]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][65]={1,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][65]={3,3,34,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[40][65]={3,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][65]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][65]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][65]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][65]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][65]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][65]={6,0,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[53][65]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][65]={6,0,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[55][65]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][65]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][65]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][65]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][65]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][65]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][65]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][65]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][65]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][65]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][65]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][65]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][65]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][65]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][65]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][65]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][65]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][65]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][65]={3,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[74][65]={3,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[75][65]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][65]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][65]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][65]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][65]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][65]={0,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[81][65]={0,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][65]={0,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][65]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][65]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][65]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][65]={0,5,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][65]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][65]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][65]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][65]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][65]={10,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][65]={10,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][65]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][65]={10,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][65]={10,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][65]={10,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][65]={2,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][65]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][65]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][65]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][65]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][65]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][65]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][65]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][65]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][65]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][65]={0,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][65]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][65]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][65]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][65]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][65]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][65]={1,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[114][65]={1,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][65]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][65]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][65]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][65]={0,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][65]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[133][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][65]={1,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][65]={1,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][65]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[177][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[180][65]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[181][65]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][65]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][65]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][65]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][65]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][65]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[198][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][65]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[203][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][65]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][65]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[5][66]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[6][66]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][66]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][66]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][66]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[22][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[23][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[24][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][66]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[27][66]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[28][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[29][66]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][66]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][66]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[46][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][66]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][66]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][66]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][66]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][66]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][66]={6,0,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][66]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][66]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][66]={3,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[56][66]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][66]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][66]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][66]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][66]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][66]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][66]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][66]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][66]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][66]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][66]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][66]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][66]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][66]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][66]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][66]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][66]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][66]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][66]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][66]={3,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[76][66]={3,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[77][66]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][66]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][66]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][66]={0,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][66]={0,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][66]={0,5,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[83][66]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][66]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][66]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][66]={10,3,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[87][66]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][66]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][66]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][66]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][66]={14,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][66]={10,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][66]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][66]={10,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][66]={11,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][66]={11,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][66]={2,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][66]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][66]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][66]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][66]={1,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[102][66]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[103][66]={0,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[104][66]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][66]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][66]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][66]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][66]={0,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[109][66]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][66]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][66]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][66]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][66]={1,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][66]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][66]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][66]={0,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][66]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][66]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[125][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][66]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][66]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][66]={1,2,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][66]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[177][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[178][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][66]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][66]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][66]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][66]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][66]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][66]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][66]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][66]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[187][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][66]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][66]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][66]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][67]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][67]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][67]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][67]={4,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[9][67]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][67]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][67]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][67]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[13][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][67]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][67]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[31][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][67]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[36][67]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][67]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][67]={4,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[48][67]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][67]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][67]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][67]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][67]={6,0,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][67]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][67]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][67]={3,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[56][67]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][67]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][67]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][67]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][67]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][67]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][67]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][67]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][67]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][67]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][67]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][67]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][67]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][67]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][67]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][67]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][67]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][67]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][67]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][67]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][67]={3,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[77][67]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][67]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][67]={0,5,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][67]={0,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][67]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][67]={1,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][67]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][67]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][67]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][67]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][67]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][67]={13,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[89][67]={14,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[90][67]={12,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[91][67]={9,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][67]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][67]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][67]={10,3,36,{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][67]={11,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[96][67]={11,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][67]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][67]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][67]={0,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[100][67]={0,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][67]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][67]={1,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][67]={2,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[104][67]={0,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[105][67]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][67]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[107][67]={0,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][67]={0,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][67]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][67]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][67]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][67]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][67]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][67]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[115][67]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[116][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][67]={0,5,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[120][67]={0,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][67]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[122][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][67]={3,2,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][67]={3,2,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][67]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][67]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][67]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[147][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][67]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[178][67]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[179][67]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][67]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][67]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][67]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][67]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][67]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][67]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][67]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[187][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][67]={3,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][67]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][67]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][67]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][68]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[6][68]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][68]={4,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[8][68]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][68]={4,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[10][68]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][68]={4,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][68]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][68]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[14][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][68]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][68]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][68]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][68]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][68]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][68]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][68]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][68]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][68]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][68]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][68]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][68]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][68]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][68]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][68]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][68]={4,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][68]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][68]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][68]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][68]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][68]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][68]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][68]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][68]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][68]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][68]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][68]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][68]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][68]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][68]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][68]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][68]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][68]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][68]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][68]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][68]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][68]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][68]={3,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[78][68]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][68]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][68]={1,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][68]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][68]={1,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][68]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][68]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][68]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][68]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][68]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][68]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][68]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][68]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[91][68]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][68]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][68]={14,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[94][68]={10,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[95][68]={1,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][68]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][68]={1,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][68]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[99][68]={0,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][68]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][68]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][68]={1,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[103][68]={1,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][68]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][68]={0,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[106][68]={0,3,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[107][68]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][68]={0,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[109][68]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][68]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][68]={0,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[112][68]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][68]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][68]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][68]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][68]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[119][68]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][68]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][68]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][68]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][68]={0,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][68]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][68]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[129][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[146][68]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[147][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[148][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[175][68]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[176][68]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][68]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][68]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][68]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][68]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][68]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][68]={4,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[184][68]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][68]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][68]={3,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][68]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][68]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][68]={0,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][68]={3,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][68]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][68]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][69]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][69]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][69]={3,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[8][69]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][69]={4,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[10][69]={4,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[11][69]={4,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][69]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][69]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[14][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][69]={4,-1,34,{{0,-1},{1,3},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[22][69]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][69]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][69]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][69]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[33][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][69]={3,-1,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[36][69]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][69]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][69]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][69]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][69]={4,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][69]={4,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][69]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][69]={1,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[48][69]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][69]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][69]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][69]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][69]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][69]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][69]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][69]={5,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[56][69]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][69]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][69]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][69]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][69]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][69]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][69]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][69]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][69]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][69]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][69]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][69]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][69]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][69]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][69]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][69]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][69]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][69]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][69]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][69]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][69]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][69]={0,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[78][69]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][69]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][69]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][69]={11,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[82][69]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][69]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][69]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][69]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][69]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][69]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][69]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][69]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][69]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][69]={14,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[92][69]={14,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[93][69]={14,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[94][69]={14,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[95][69]={10,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[96][69]={1,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[97][69]={0,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][69]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][69]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][69]={1,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][69]={1,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][69]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][69]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][69]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][69]={2,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][69]={0,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[107][69]={0,5,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[108][69]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[109][69]={0,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[110][69]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][69]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][69]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][69]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][69]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][69]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[119][69]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][69]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[125][69]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][69]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][69]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[129][69]={4,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[130][69]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[146][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[147][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][69]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[175][69]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][69]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][69]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][69]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][69]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][69]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][69]={4,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[184][69]={4,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][69]={3,-1,17,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[186][69]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][69]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][69]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[189][69]={3,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[190][69]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][69]={0,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[201][69]={0,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[202][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][69]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][69]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][70]={3,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[6][70]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][70]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][70]={4,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[9][70]={4,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[10][70]={3,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][70]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][70]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][70]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][70]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][70]={3,3,34,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[23][70]={4,-1,34,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][70]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][70]={3,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[31][70]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][70]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][70]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][70]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][70]={3,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][70]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[38][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][70]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][70]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][70]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][70]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][70]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][70]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][70]={2,-1,33,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[48][70]={2,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[49][70]={2,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][70]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][70]={4,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[52][70]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][70]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][70]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][70]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][70]={5,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][70]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][70]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][70]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][70]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][70]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][70]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][70]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][70]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][70]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][70]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][70]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][70]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][70]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][70]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][70]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][70]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][70]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][70]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][70]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][70]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][70]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][70]={1,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[79][70]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][70]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][70]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][70]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][70]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][70]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][70]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][70]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][70]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][70]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][70]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[90][70]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][70]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][70]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][70]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][70]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][70]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][70]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][70]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][70]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][70]={1,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][70]={1,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[101][70]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][70]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][70]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][70]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][70]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][70]={0,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][70]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][70]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][70]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][70]={1,-1,36,{{0,-1},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[111][70]={0,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[112][70]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[113][70]={0,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[114][70]={0,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[115][70]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][70]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][70]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[123][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][70]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][70]={3,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][70]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][70]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[178][70]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][70]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][70]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][70]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][70]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][70]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][70]={6,0,17,{{0,-1},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[185][70]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][70]={4,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[187][70]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][70]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][70]={3,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][70]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][70]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][70]={0,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][70]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][70]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][70]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][70]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][70]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][70]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][71]={3,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[6][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][71]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][71]={1,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][71]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][71]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][71]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][71]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[28][71]={4,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{1,0,0}}
 MapToConvert[29][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][71]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][71]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][71]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][71]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][71]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][71]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[40][71]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][71]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][71]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][71]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][71]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][71]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][71]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][71]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][71]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][71]={4,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[52][71]={4,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[53][71]={3,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][71]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][71]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][71]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[61][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[62][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][71]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][71]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][71]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][71]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][71]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][71]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][71]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][71]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][71]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][71]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][71]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][71]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][71]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][71]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][71]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][71]={11,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[79][71]={9,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[80][71]={9,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[81][71]={11,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[82][71]={11,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[83][71]={9,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[84][71]={9,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[85][71]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][71]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][71]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][71]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][71]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[90][71]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][71]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][71]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][71]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][71]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][71]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][71]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][71]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][71]={5,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][71]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][71]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][71]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][71]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][71]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][71]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][71]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][71]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][71]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][71]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][71]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[110][71]={0,5,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][71]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][71]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][71]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][71]={0,3,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[115][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][71]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][71]={1,3,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[122][71]={3,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][71]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][71]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][71]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[126][71]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][71]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][71]={4,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[130][71]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][71]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][71]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][71]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[178][71]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][71]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][71]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][71]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][71]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][71]={7,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][71]={7,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][71]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][71]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][71]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][71]={3,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][71]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][71]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][71]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][71]={0,3,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[193][71]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][71]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][71]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[196][71]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[197][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][71]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][71]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][71]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][71]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][71]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][71]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[5][72]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[6][72]={4,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[7][72]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[8][72]={0,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[9][72]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[10][72]={1,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][72]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][72]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][72]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][72]={4,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][72]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][72]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][72]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][72]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][72]={4,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[34][72]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][72]={3,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[36][72]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[37][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][72]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[39][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[41][72]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][72]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][72]={4,3,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[44][72]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][72]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][72]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][72]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][72]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][72]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][72]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][72]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][72]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][72]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][72]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][72]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][72]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][72]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][72]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[59][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][72]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][72]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][72]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][72]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][72]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][72]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][72]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][72]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][72]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][72]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][72]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][72]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][72]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][72]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][72]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][72]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][72]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][72]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][72]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][72]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][72]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][72]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][72]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][72]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][72]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][72]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][72]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][72]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][72]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][72]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][72]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][72]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][72]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][72]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][72]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][72]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][72]={4,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][72]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][72]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][72]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][72]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][72]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][72]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][72]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][72]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][72]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][72]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][72]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][72]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][72]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][72]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][72]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[122][72]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][72]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][72]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][72]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][72]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][72]={1,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][72]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][72]={4,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][72]={0,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][72]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][72]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[134][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[174][72]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[175][72]={4,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][72]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][72]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][72]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][72]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][72]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][72]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][72]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][72]={8,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][72]={7,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][72]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][72]={4,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][72]={3,3,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[188][72]={3,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][72]={3,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][72]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][72]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][72]={0,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][72]={0,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][72]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][72]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][72]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][72]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][72]={0,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[199][72]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][72]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][72]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][72]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][72]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][72]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[5][73]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][73]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][73]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][73]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][73]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][73]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][73]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][73]={0,5,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[13][73]={0,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][73]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][73]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][73]={0,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[26][73]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][73]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][73]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][73]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[30][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][73]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][73]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][73]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][73]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][73]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][73]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][73]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[40][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][73]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][73]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][73]={1,-1,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[44][73]={4,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][73]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][73]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][73]={3,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][73]={1,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[49][73]={3,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[50][73]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][73]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][73]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][73]={5,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][73]={4,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][73]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][73]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][73]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][73]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[59][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][73]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][73]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][73]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][73]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][73]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][73]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][73]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][73]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][73]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][73]={4,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][73]={10,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[74][73]={10,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[75][73]={10,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[76][73]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][73]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][73]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][73]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][73]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][73]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][73]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][73]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][73]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][73]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][73]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][73]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][73]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][73]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][73]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][73]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][73]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][73]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][73]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][73]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][73]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][73]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][73]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][73]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][73]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][73]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][73]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][73]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][73]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][73]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][73]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][73]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][73]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][73]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][73]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][73]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][73]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[114][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][73]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[119][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[122][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[123][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[124][73]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][73]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][73]={1,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][73]={3,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][73]={1,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][73]={0,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][73]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][73]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][73]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][73]={4,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][73]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][73]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][73]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][73]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][73]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][73]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][73]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][73]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][73]={7,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[184][73]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][73]={4,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][73]={4,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][73]={3,3,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[188][73]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][73]={3,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][73]={0,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[191][73]={0,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[192][73]={0,-1,17,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[193][73]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][73]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][73]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][73]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][73]={0,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][73]={4,3,17,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[199][73]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][73]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][73]={0,5,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][73]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[203][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][73]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][73]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][74]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[6][74]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][74]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][74]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][74]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][74]={1,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[11][74]={1,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[12][74]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][74]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][74]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][74]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][74]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[17][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][74]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[21][74]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[22][74]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[23][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][74]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][74]={4,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][74]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][74]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][74]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][74]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][74]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][74]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][74]={0,3,33,{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][74]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][74]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][74]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[41][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][74]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][74]={1,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][74]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][74]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][74]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][74]={1,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][74]={1,3,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[49][74]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][74]={3,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][74]={3,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[52][74]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][74]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][74]={4,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[55][74]={1,-1,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[56][74]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][74]={4,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][74]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[59][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][74]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][74]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][74]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][74]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][74]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][74]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][74]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][74]={6,0,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][74]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][74]={4,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][74]={10,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][74]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][74]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][74]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][74]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][74]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][74]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][74]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][74]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][74]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][74]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][74]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][74]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][74]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][74]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][74]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][74]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][74]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][74]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][74]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][74]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][74]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][74]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[96][74]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][74]={3,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[98][74]={4,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[99][74]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][74]={3,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[101][74]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][74]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][74]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][74]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][74]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][74]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][74]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][74]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][74]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][74]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][74]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][74]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][74]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][74]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][74]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[120][74]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[121][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[125][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[126][74]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][74]={2,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[128][74]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][74]={1,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[130][74]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][74]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][74]={3,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][74]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[134][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][74]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][74]={4,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[176][74]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][74]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][74]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][74]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][74]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][74]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][74]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][74]={7,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][74]={7,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][74]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][74]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][74]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][74]={3,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][74]={3,3,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[190][74]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][74]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][74]={0,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][74]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][74]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][74]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][74]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][74]={4,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][74]={4,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][74]={3,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[200][74]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][74]={0,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][74]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][74]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[204][74]={0,5,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][74]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][74]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][74]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][75]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][75]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][75]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][75]={1,3,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[8][75]={1,3,33,{{1,2},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[9][75]={1,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][75]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][75]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][75]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][75]={2,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][75]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][75]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][75]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][75]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][75]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][75]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][75]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[24][75]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[25][75]={0,5,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[26][75]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][75]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][75]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][75]={1,3,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[33][75]={1,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[34][75]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][75]={1,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][75]={1,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][75]={1,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[38][75]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][75]={1,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][75]={1,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[45][75]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][75]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][75]={4,3,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[48][75]={4,-1,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[49][75]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][75]={3,3,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[51][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[52][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[54][75]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][75]={1,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[56][75]={0,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[57][75]={5,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][75]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][75]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][75]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][75]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][75]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][75]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][75]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][75]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][75]={6,0,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][75]={6,0,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[71][75]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[72][75]={3,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][75]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][75]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][75]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][75]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][75]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][75]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][75]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][75]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][75]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][75]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][75]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][75]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][75]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][75]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][75]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][75]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][75]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][75]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][75]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][75]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][75]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][75]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[95][75]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[96][75]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[97][75]={3,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[98][75]={4,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][75]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][75]={3,3,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[101][75]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[102][75]={4,-1,36,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[103][75]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[104][75]={3,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[105][75]={3,-1,36,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[106][75]={5,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[107][75]={4,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[108][75]={4,3,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[109][75]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][75]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][75]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][75]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][75]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][75]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][75]={1,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][75]={1,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][75]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[126][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][75]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][75]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][75]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][75]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][75]={3,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][75]={4,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[133][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[134][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][75]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][75]={3,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][75]={4,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][75]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][75]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][75]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][75]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][75]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][75]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][75]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][75]={7,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[184][75]={7,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][75]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][75]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[187][75]={3,3,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[188][75]={3,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][75]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][75]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][75]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][75]={0,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[193][75]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][75]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][75]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][75]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][75]={3,3,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[198][75]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][75]={5,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[200][75]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][75]={4,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][75]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][75]={0,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][75]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][75]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][75]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][75]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[5][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[8][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[9][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[10][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[11][76]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][76]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][76]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][76]={0,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[15][76]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][76]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][76]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][76]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][76]={1,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[20][76]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][76]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][76]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][76]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][76]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][76]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][76]={0,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][76]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][76]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][76]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][76]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][76]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][76]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][76]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][76]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][76]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][76]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][76]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[46][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[47][76]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][76]={4,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[49][76]={4,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[50][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][76]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][76]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][76]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][76]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][76]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[59][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][76]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][76]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][76]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][76]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][76]={6,0,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][76]={6,0,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[69][76]={6,0,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[70][76]={6,0,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][76]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][76]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][76]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][76]={10,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][76]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[76][76]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[77][76]={10,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[78][76]={10,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[79][76]={11,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[80][76]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][76]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][76]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][76]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][76]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][76]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][76]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][76]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][76]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][76]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][76]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][76]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][76]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][76]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][76]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][76]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][76]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][76]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][76]={4,3,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[99][76]={3,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[100][76]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][76]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][76]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][76]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][76]={3,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][76]={5,-1,36,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[106][76]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][76]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][76]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][76]={0,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[110][76]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][76]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][76]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][76]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][76]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][76]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[116][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][76]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][76]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][76]={1,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[121][76]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[127][76]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][76]={2,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][76]={1,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][76]={1,3,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[131][76]={4,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[132][76]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][76]={3,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[174][76]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][76]={4,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[176][76]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][76]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][76]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][76]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][76]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][76]={8,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][76]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][76]={4,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][76]={4,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][76]={4,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][76]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][76]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][76]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][76]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][76]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][76]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][76]={0,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][76]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][76]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][76]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][76]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][76]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][76]={4,3,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[199][76]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][76]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][76]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][76]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][76]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][76]={0,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][76]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][76]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][76]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][76]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][77]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][77]={0,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][77]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[14][77]={0,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][77]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][77]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][77]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][77]={0,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][77]={1,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][77]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][77]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][77]={4,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[23][77]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][77]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][77]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][77]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[29][77]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[30][77]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][77]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][77]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][77]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][77]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][77]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][77]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][77]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][77]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][77]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][77]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[41][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[47][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][77]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][77]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][77]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][77]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][77]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[58][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[59][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][77]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][77]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][77]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][77]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][77]={6,0,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][77]={6,0,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][77]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][77]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][77]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][77]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][77]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][77]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][77]={10,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][77]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][77]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][77]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][77]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][77]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][77]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][77]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][77]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][77]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][77]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][77]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][77]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][77]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][77]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][77]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][77]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][77]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][77]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][77]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][77]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][77]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][77]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][77]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][77]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][77]={3,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[100][77]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][77]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][77]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][77]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][77]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][77]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][77]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][77]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][77]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][77]={0,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[110][77]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[111][77]={0,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[112][77]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][77]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[115][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[116][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][77]={0,5,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[119][77]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][77]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][77]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[122][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[127][77]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[129][77]={1,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[130][77]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][77]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][77]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[133][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][77]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][77]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][77]={4,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[176][77]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][77]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][77]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][77]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][77]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][77]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][77]={8,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][77]={4,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][77]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][77]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][77]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][77]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][77]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][77]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][77]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][77]={0,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][77]={0,5,17,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[193][77]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][77]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][77]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][77]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][77]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][77]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][77]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][77]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][77]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][77]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][77]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][77]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][77]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][77]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[207][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][77]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][77]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][78]={3,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][78]={3,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[14][78]={0,3,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[15][78]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][78]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][78]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][78]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][78]={0,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[20][78]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][78]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][78]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][78]={4,3,33,{{0,-1},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[24][78]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[25][78]={4,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[26][78]={3,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[27][78]={0,5,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][78]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][78]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][78]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][78]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][78]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][78]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][78]={1,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][78]={1,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[38][78]={0,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[39][78]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[40][78]={0,-1,33,{{1,5},{1,0},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[41][78]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[42][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][78]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][78]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][78]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][78]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][78]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][78]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][78]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][78]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][78]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][78]={6,0,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][78]={6,0,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][78]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][78]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][78]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][78]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][78]={6,0,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][78]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[74][78]={10,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][78]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][78]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][78]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][78]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][78]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][78]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][78]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][78]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][78]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][78]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][78]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][78]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][78]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][78]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][78]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][78]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][78]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][78]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][78]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][78]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][78]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][78]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][78]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][78]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][78]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][78]={3,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[101][78]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[102][78]={3,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[103][78]={4,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[104][78]={4,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][78]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][78]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][78]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][78]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][78]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][78]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][78]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][78]={0,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[113][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[116][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[118][78]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][78]={1,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[120][78]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][78]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[122][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[123][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[128][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[129][78]={2,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][78]={1,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][78]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][78]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][78]={4,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][78]={3,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[175][78]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][78]={4,-1,17,{{0,-1},{1,0},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[177][78]={6,0,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][78]={6,0,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][78]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][78]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][78]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][78]={8,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[183][78]={4,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][78]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][78]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][78]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][78]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][78]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][78]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][78]={1,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][78]={0,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[192][78]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][78]={0,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[194][78]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][78]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][78]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][78]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][78]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][78]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][78]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][78]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][78]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][78]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][78]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][78]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][78]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][78]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][78]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][79]={3,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][79]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][79]={1,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[15][79]={4,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[16][79]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][79]={4,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[18][79]={1,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][79]={1,3,33,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[20][79]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][79]={5,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[22][79]={4,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[23][79]={4,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][79]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][79]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][79]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][79]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][79]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][79]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][79]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][79]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][79]={1,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][79]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[34][79]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[35][79]={3,-1,33,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[36][79]={1,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][79]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][79]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][79]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][79]={1,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][79]={0,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[47][79]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][79]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[52][79]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][79]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][79]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][79]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][79]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][79]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[58][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[61][79]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][79]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][79]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][79]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][79]={6,0,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][79]={6,0,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][79]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][79]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][79]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][79]={6,0,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][79]={6,0,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][79]={6,0,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][79]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][79]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][79]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][79]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][79]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][79]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][79]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][79]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][79]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][79]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][79]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][79]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][79]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][79]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][79]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][79]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][79]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][79]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][79]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][79]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][79]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][79]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][79]={5,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][79]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][79]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][79]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][79]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][79]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][79]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][79]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][79]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][79]={0,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[120][79]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[129][79]={1,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][79]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][79]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][79]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[133][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][79]={4,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[174][79]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][79]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][79]={4,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[177][79]={7,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[178][79]={6,0,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[179][79]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][79]={6,0,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][79]={6,0,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][79]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][79]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][79]={4,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[185][79]={4,3,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[186][79]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[187][79]={3,3,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[188][79]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[189][79]={4,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[190][79]={0,5,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][79]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][79]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][79]={0,3,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[194][79]={3,-1,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[195][79]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][79]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][79]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][79]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][79]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][79]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][79]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][79]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][79]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][79]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][79]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][79]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][79]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][80]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][80]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][80]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][80]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][80]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][80]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][80]={4,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[19][80]={3,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[20][80]={3,3,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[21][80]={4,-1,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[22][80]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][80]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][80]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][80]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][80]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][80]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][80]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][80]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][80]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][80]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][80]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][80]={3,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[34][80]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][80]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][80]={3,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][80]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][80]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][80]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][80]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][80]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][80]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[47][80]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[48][80]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][80]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][80]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][80]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][80]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][80]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][80]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][80]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][80]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][80]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[59][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[62][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[63][80]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][80]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][80]={6,0,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][80]={6,0,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][80]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][80]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][80]={6,0,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][80]={6,0,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[71][80]={6,0,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[72][80]={6,0,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][80]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][80]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][80]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][80]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][80]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][80]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][80]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][80]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][80]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][80]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][80]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][80]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][80]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][80]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][80]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][80]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][80]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][80]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][80]={4,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[107][80]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][80]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][80]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][80]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][80]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][80]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[113][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[115][80]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[116][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][80]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][80]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][80]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[129][80]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[130][80]={1,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][80]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][80]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[134][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[173][80]={1,3,17,{{0,-1},{1,3},{1,4}},{-1,1},{1,0,0}}
 MapToConvert[174][80]={3,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][80]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][80]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][80]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][80]={6,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][80]={6,0,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[180][80]={6,0,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[181][80]={7,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[182][80]={5,-1,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[183][80]={4,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[184][80]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][80]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][80]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][80]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][80]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][80]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][80]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][80]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][80]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][80]={0,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][80]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][80]={3,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[196][80]={3,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[197][80]={3,-1,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[198][80]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][80]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][80]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][80]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][80]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][80]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][80]={1,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[205][80]={0,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[206][80]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][80]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][80]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[11][81]={0,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{1,0,0}}
 MapToConvert[12][81]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[13][81]={0,3,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[14][81]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][81]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][81]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][81]={4,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][81]={3,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][81]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][81]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][81]={4,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[22][81]={5,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][81]={5,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[24][81]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][81]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][81]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][81]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][81]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][81]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][81]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][81]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][81]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][81]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][81]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][81]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][81]={0,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][81]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][81]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][81]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][81]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][81]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][81]={1,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][81]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[44][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][81]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][81]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][81]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][81]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][81]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][81]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][81]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][81]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][81]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][81]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][81]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][81]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][81]={6,0,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[66][81]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][81]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][81]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][81]={6,0,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][81]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][81]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][81]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][81]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][81]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[81][81]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][81]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][81]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][81]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][81]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][81]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][81]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][81]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][81]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][81]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][81]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][81]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][81]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][81]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][81]={4,-1,36,{{0,-1},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[107][81]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[108][81]={4,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[109][81]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][81]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][81]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][81]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][81]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[116][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][81]={0,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[119][81]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][81]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][81]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[122][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[129][81]={1,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][81]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][81]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][81]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][81]={1,3,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[174][81]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][81]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][81]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][81]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][81]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][81]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][81]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][81]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][81]={5,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[183][81]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][81]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][81]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][81]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][81]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][81]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][81]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][81]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][81]={0,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][81]={0,3,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[193][81]={0,-1,17,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[194][81]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][81]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][81]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][81]={3,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[198][81]={3,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[199][81]={0,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[200][81]={0,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[201][81]={0,-1,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[202][81]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][81]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][81]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][81]={1,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[206][81]={1,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][81]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][81]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[9][82]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][82]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][82]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][82]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][82]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][82]={0,3,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[15][82]={0,-1,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[16][82]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][82]={4,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][82]={3,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][82]={3,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][82]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][82]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][82]={1,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][82]={1,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[24][82]={1,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][82]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][82]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][82]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][82]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][82]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][82]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][82]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][82]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[33][82]={4,-1,33,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[34][82]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][82]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][82]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][82]={0,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[38][82]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][82]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][82]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][82]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][82]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][82]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][82]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][82]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][82]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][82]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][82]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][82]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][82]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][82]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][82]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][82]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][82]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[61][82]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][82]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][82]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][82]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][82]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][82]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][82]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][82]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][82]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][82]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[79][82]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[81][82]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][82]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][82]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][82]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][82]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][82]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][82]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][82]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][82]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][82]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][82]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][82]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][82]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][82]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][82]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][82]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][82]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][82]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][82]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][82]={3,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[110][82]={4,-1,36,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[111][82]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[112][82]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][82]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][82]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][82]={0,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[117][82]={0,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][82]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][82]={4,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[120][82]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][82]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][82]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[123][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[129][82]={1,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[130][82]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][82]={3,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][82]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][82]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][82]={4,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[175][82]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][82]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][82]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][82]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][82]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][82]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][82]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][82]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][82]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][82]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][82]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][82]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][82]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][82]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][82]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][82]={0,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][82]={0,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[192][82]={0,5,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][82]={0,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][82]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][82]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][82]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][82]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][82]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][82]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][82]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][82]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][82]={1,3,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[203][82]={1,-1,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[204][82]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][82]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][82]={1,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][82]={1,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][82]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][82]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][82]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[9][83]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][83]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][83]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][83]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][83]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][83]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][83]={0,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[16][83]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[17][83]={3,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][83]={3,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][83]={3,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][83]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][83]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][83]={1,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[23][83]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][83]={1,-1,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[25][83]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][83]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][83]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][83]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][83]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][83]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[31][83]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][83]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][83]={1,3,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[34][83]={1,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][83]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][83]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][83]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][83]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][83]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][83]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][83]={1,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][83]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][83]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][83]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][83]={3,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[46][83]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][83]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][83]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][83]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][83]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][83]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][83]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][83]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][83]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][83]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][83]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][83]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][83]={6,0,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][83]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][83]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][83]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][83]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][83]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][83]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][83]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][83]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][83]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][83]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][83]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][83]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][83]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][83]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][83]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][83]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][83]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][83]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][83]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][83]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][83]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][83]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][83]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][83]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][83]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][83]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][83]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][83]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][83]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][83]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][83]={4,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[111][83]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][83]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][83]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][83]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][83]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][83]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][83]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][83]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][83]={10,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[120][83]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][83]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][83]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[123][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[129][83]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][83]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][83]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][83]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][83]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][83]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][83]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][83]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][83]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][83]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][83]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][83]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][83]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][83]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][83]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][83]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][83]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][83]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][83]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][83]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][83]={3,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][83]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][83]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][83]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][83]={0,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[194][83]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][83]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][83]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][83]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][83]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][83]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][83]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][83]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][83]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][83]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][83]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][83]={1,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][83]={1,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][83]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][83]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][83]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][83]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[9][84]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][84]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][84]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][84]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][84]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][84]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][84]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][84]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][84]={0,-1,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[18][84]={3,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][84]={3,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][84]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][84]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][84]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][84]={1,3,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[24][84]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][84]={4,3,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[26][84]={3,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[27][84]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[28][84]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[29][84]={1,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[30][84]={1,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][84]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][84]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][84]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][84]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][84]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][84]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][84]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][84]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][84]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][84]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][84]={1,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][84]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][84]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][84]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][84]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][84]={3,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[47][84]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][84]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][84]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][84]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][84]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][84]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[53][84]={3,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{1,0,0}}
 MapToConvert[54][84]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][84]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][84]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][84]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[58][84]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][84]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][84]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][84]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][84]={6,0,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[63][84]={6,0,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][84]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][84]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][84]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][84]={6,4,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][84]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][84]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][84]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[78][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][84]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][84]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][84]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][84]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][84]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][84]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][84]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][84]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][84]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][84]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][84]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][84]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][84]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][84]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][84]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][84]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][84]={4,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[111][84]={3,3,36,{{1,2},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[112][84]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][84]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][84]={3,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][84]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][84]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][84]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][84]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][84]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][84]={10,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[121][84]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][84]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][84]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[124][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][84]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][84]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][84]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][84]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][84]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][84]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][84]={7,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][84]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][84]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][84]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][84]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][84]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][84]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][84]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][84]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][84]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][84]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][84]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][84]={0,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][84]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][84]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][84]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][84]={0,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][84]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][84]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][84]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[197][84]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][84]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][84]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][84]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][84]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[202][84]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][84]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][84]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][84]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][84]={1,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[207][84]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][84]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][84]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[210][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][84]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][84]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[9][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[10][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[12][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][85]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][85]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][85]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][85]={0,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][85]={0,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[18][85]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][85]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][85]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][85]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][85]={4,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][85]={4,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][85]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][85]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][85]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][85]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][85]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][85]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][85]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][85]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][85]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][85]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][85]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][85]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][85]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][85]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][85]={1,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][85]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[40][85]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][85]={0,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][85]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][85]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][85]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][85]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][85]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][85]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][85]={3,3,33,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[49][85]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][85]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][85]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][85]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][85]={3,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[54][85]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][85]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][85]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][85]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][85]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][85]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][85]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][85]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][85]={3,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[63][85]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][85]={7,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][85]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][85]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][85]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][85]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][85]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][85]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][85]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][85]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][85]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][85]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][85]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][85]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][85]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][85]={6,0,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][85]={6,0,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][85]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][85]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][85]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][85]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][85]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][85]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][85]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][85]={0,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][85]={0,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][85]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][85]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][85]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][85]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][85]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][85]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][85]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[125][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[130][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[131][85]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][85]={1,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[173][85]={1,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[174][85]={1,3,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[175][85]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][85]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][85]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][85]={2,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[179][85]={2,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][85]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][85]={4,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][85]={4,3,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[183][85]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][85]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][85]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][85]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][85]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][85]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][85]={0,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[190][85]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][85]={0,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][85]={0,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[193][85]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][85]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][85]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][85]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[197][85]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][85]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][85]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][85]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][85]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[202][85]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][85]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][85]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][85]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][85]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][85]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][85]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][85]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][85]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][85]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[214][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][85]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][85]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[8][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[9][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[13][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[15][86]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[16][86]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][86]={4,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][86]={3,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][86]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][86]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][86]={4,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][86]={3,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][86]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][86]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][86]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][86]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][86]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][86]={1,3,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[39][86]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][86]={0,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][86]={3,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[46][86]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][86]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][86]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][86]={3,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[50][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][86]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][86]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][86]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][86]={3,3,33,{{1,2},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][86]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[56][86]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][86]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][86]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][86]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][86]={4,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][86]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][86]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][86]={6,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][86]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][86]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][86]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][86]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[80][86]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[81][86]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][86]={6,0,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][86]={6,0,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][86]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][86]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][86]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][86]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][86]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][86]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][86]={3,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[99][86]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][86]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][86]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][86]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][86]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][86]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][86]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][86]={4,-1,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[112][86]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][86]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][86]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][86]={1,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][86]={1,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[117][86]={0,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][86]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][86]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][86]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][86]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][86]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][86]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][86]={11,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][86]={0,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][86]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[130][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[132][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[133][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[173][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[174][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][86]={1,3,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[176][86]={1,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[177][86]={1,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[178][86]={2,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][86]={1,3,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[180][86]={1,3,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][86]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][86]={4,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][86]={3,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[184][86]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[185][86]={3,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[186][86]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][86]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][86]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][86]={0,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][86]={3,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][86]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][86]={0,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[193][86]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][86]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][86]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][86]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][86]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][86]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][86]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][86]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][86]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][86]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][86]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][86]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][86]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][86]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][86]={1,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[208][86]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][86]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][86]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][86]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][86]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][86]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][86]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[8][87]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[9][87]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][87]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[13][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[14][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[15][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][87]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][87]={0,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][87]={3,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][87]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][87]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][87]={3,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][87]={3,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][87]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][87]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][87]={0,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[30][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][87]={0,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[32][87]={0,3,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[33][87]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][87]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][87]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][87]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][87]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][87]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][87]={0,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][87]={3,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[44][87]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][87]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][87]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][87]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][87]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][87]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][87]={3,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[53][87]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][87]={3,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[55][87]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][87]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][87]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][87]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][87]={0,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[63][87]={1,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[64][87]={4,3,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[65][87]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][87]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][87]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][87]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][87]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][87]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][87]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][87]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[77][87]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[78][87]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][87]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[81][87]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][87]={6,0,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][87]={6,0,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][87]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][87]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][87]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][87]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][87]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][87]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][87]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][87]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][87]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][87]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][87]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][87]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][87]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][87]={7,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][87]={3,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[99][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][87]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][87]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][87]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][87]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][87]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][87]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][87]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][87]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][87]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][87]={3,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[116][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][87]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][87]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][87]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][87]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][87]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][87]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][87]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][87]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][87]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[127][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[129][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][87]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][87]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[132][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][87]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][87]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][87]={0,3,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[176][87]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][87]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][87]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][87]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][87]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][87]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][87]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][87]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][87]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][87]={3,3,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[186][87]={3,-1,17,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[187][87]={0,3,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[188][87]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[189][87]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][87]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][87]={0,5,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[192][87]={0,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][87]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][87]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][87]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[196][87]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][87]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][87]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[199][87]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][87]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][87]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][87]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][87]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][87]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][87]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][87]={0,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][87]={1,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[208][87]={0,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[209][87]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][87]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][87]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][87]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][87]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][87]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][87]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][88]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][88]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][88]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][88]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[20][88]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[21][88]={0,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[22][88]={3,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][88]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][88]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][88]={0,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][88]={3,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][88]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][88]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][88]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][88]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][88]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][88]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][88]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][88]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][88]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][88]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][88]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][88]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][88]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][88]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][88]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][88]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][88]={4,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][88]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][88]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][88]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][88]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][88]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][88]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][88]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][88]={3,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[52][88]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][88]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][88]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][88]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][88]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][88]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][88]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][88]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][88]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][88]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][88]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][88]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][88]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][88]={1,3,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[66][88]={4,-1,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[67][88]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][88]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][88]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][88]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][88]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][88]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][88]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][88]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][88]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][88]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][88]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][88]={6,0,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][88]={6,0,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][88]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][88]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][88]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][88]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][88]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][88]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][88]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][88]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][88]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][88]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][88]={11,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[97][88]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[98][88]={4,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[99][88]={3,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[100][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][88]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][88]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][88]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][88]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][88]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][88]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][88]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][88]={3,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][88]={3,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[117][88]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[118][88]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[119][88]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[120][88]={0,5,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[121][88]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][88]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][88]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][88]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][88]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][88]={1,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[130][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][88]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][88]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][88]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][88]={1,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[176][88]={0,3,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[177][88]={0,3,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[178][88]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][88]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][88]={4,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[181][88]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][88]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][88]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][88]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][88]={3,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][88]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[187][88]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][88]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][88]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][88]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][88]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][88]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][88]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][88]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][88]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][88]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][88]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][88]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][88]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][88]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][88]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][88]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][88]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][88]={0,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][88]={0,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[206][88]={0,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[207][88]={0,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][88]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][88]={0,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[210][88]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][88]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][88]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[213][88]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[214][88]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][88]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][88]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][88]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][88]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][89]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[12][89]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[13][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[14][89]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][89]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[16][89]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[17][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][89]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][89]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][89]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][89]={0,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[26][89]={0,5,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][89]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[28][89]={0,3,33,{{1,5},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][89]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][89]={0,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[33][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][89]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][89]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][89]={3,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][89]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][89]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][89]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][89]={0,5,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[42][89]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][89]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][89]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][89]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][89]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][89]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][89]={0,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][89]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][89]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][89]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][89]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][89]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][89]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][89]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][89]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][89]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][89]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][89]={10,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[67][89]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][89]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][89]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][89]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][89]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][89]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][89]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][89]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][89]={6,0,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][89]={6,0,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][89]={6,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][89]={6,4,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][89]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][89]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][89]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][89]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][89]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][89]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][89]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][89]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][89]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][89]={3,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[100][89]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][89]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][89]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][89]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][89]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][89]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][89]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][89]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][89]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][89]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][89]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][89]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][89]={3,3,36,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[116][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][89]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][89]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][89]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][89]={1,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[121][89]={0,5,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[122][89]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][89]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][89]={9,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[125][89]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][89]={10,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[127][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][89]={4,-1,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[132][89]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][89]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[134][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[172][89]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][89]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][89]={1,3,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[176][89]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][89]={9,3,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[178][89]={11,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][89]={4,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][89]={4,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][89]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][89]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][89]={3,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][89]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[185][89]={3,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][89]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][89]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][89]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][89]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][89]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][89]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][89]={0,5,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][89]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][89]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][89]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[196][89]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][89]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][89]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][89]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][89]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][89]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][89]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][89]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][89]={0,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[205][89]={9,3,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[206][89]={1,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][89]={1,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[208][89]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][89]={0,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[210][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][89]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][89]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][89]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[217][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][89]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][89]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][89]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[10][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[11][90]={0,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[12][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][90]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[18][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][90]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][90]={0,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][90]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][90]={0,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][90]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[27][90]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][90]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][90]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][90]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][90]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][90]={0,5,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][90]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][90]={3,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][90]={3,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][90]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][90]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][90]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][90]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][90]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][90]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][90]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][90]={1,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][90]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][90]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][90]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[50][90]={0,5,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][90]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][90]={0,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[55][90]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][90]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][90]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][90]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][90]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][90]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][90]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][90]={11,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[68][90]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][90]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][90]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][90]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][90]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][90]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][90]={0,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][90]={0,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[76][90]={0,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[77][90]={0,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[78][90]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[79][90]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[80][90]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[81][90]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[82][90]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][90]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][90]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][90]={11,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][90]={11,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][90]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][90]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][90]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][90]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][90]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][90]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][90]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][90]={4,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[101][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][90]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][90]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][90]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][90]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][90]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][90]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][90]={10,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][90]={10,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[113][90]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][90]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][90]={3,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[117][90]={3,3,36,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[118][90]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][90]={1,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][90]={1,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][90]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[122][90]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[123][90]={0,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[124][90]={0,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[125][90]={11,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][90]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][90]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][90]={4,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[133][90]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][90]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[135][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[173][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,0}}
 MapToConvert[174][90]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][90]={1,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][90]={1,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][90]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][90]={4,3,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[179][90]={4,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][90]={4,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[181][90]={3,3,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[182][90]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[183][90]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[184][90]={3,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][90]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][90]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][90]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][90]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][90]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][90]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][90]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][90]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][90]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][90]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][90]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][90]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[197][90]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][90]={0,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[199][90]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][90]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][90]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][90]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][90]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][90]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][90]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][90]={9,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[207][90]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][90]={9,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][90]={9,3,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[210][90]={4,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][90]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[218][90]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][90]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[220][90]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][90]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][90]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][90]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][90]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[3][91]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[4][91]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][91]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][91]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[7][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[9][91]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][91]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][91]={1,3,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[12][91]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][91]={1,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[14][91]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[15][91]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[16][91]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][91]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][91]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][91]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][91]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][91]={0,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][91]={0,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[29][91]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][91]={0,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[33][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][91]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][91]={3,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][91]={3,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][91]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][91]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][91]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][91]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][91]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][91]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][91]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][91]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][91]={0,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[47][91]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][91]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][91]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][91]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][91]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][91]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][91]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][91]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][91]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][91]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][91]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][91]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][91]={11,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[68][91]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][91]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][91]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][91]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][91]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][91]={0,5,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][91]={0,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][91]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][91]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][91]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][91]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][91]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][91]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][91]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][91]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][91]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][91]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][91]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][91]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][91]={11,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][91]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][91]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[90][91]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][91]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][91]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][91]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][91]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][91]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][91]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][91]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][91]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][91]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][91]={4,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[101][91]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][91]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][91]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][91]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][91]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][91]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][91]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][91]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][91]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][91]={4,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][91]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][91]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][91]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][91]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][91]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][91]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][91]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][91]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][91]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[120][91]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][91]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][91]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][91]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][91]={9,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[125][91]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][91]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[127][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[130][91]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][91]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[135][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][91]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][91]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[172][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][91]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[174][91]={0,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][91]={0,3,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][91]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][91]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][91]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][91]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][91]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][91]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][91]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][91]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][91]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][91]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][91]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][91]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][91]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][91]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][91]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][91]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][91]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][91]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][91]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][91]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][91]={9,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[197][91]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][91]={9,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[199][91]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][91]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][91]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][91]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][91]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][91]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][91]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][91]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][91]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][91]={9,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[209][91]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][91]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][91]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[214][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[218][91]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][91]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][91]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][91]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[222][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][91]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][91]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[4][92]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][92]={1,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[12][92]={1,-1,33,{{1,2},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][92]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][92]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][92]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][92]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[28][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[33][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][92]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][92]={3,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][92]={3,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][92]={3,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[38][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][92]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][92]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][92]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][92]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][92]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][92]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][92]={0,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[48][92]={0,-1,33,{{1,2},{1,3},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[49][92]={0,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[50][92]={0,-1,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[51][92]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][92]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][92]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][92]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][92]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][92]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][92]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][92]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][92]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][92]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][92]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][92]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][92]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][92]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][92]={0,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][92]={0,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][92]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][92]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][92]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][92]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][92]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][92]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][92]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][92]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][92]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][92]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][92]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][92]={3,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][92]={3,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][92]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][92]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][92]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[91][92]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[92][92]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][92]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][92]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][92]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[96][92]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[97][92]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][92]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][92]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][92]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][92]={4,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][92]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][92]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][92]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][92]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][92]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][92]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][92]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][92]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][92]={4,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][92]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][92]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][92]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][92]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][92]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][92]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][92]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][92]={3,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][92]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][92]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][92]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][92]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][92]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][92]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][92]={9,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[126][92]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[128][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][92]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[130][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[131][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[135][92]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][92]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][92]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,0}}
 MapToConvert[173][92]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[174][92]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][92]={2,-1,17,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[176][92]={2,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][92]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][92]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][92]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][92]={3,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[181][92]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[182][92]={3,3,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[183][92]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][92]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][92]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][92]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][92]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][92]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][92]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][92]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][92]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][92]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][92]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][92]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][92]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][92]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][92]={0,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[198][92]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][92]={9,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[200][92]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][92]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][92]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][92]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][92]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][92]={9,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][92]={9,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[207][92]={9,3,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][92]={9,-1,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][92]={10,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][92]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][92]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[219][92]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][92]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[223][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][92]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][92]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[4][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][93]={0,-1,33,{{1,5},{1,0},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[6][93]={3,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[7][93]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][93]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][93]={1,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[13][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][93]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][93]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][93]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][93]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[27][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[33][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[34][93]={0,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][93]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][93]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][93]={0,5,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[40][93]={0,5,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[41][93]={0,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][93]={0,3,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[43][93]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][93]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][93]={0,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[47][93]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[48][93]={0,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][93]={0,3,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[51][93]={0,3,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[52][93]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][93]={0,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][93]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][93]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][93]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][93]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][93]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][93]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][93]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][93]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][93]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][93]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][93]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][93]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][93]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][93]={0,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][93]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][93]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][93]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][93]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][93]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][93]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][93]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][93]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][93]={0,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][93]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[83][93]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[84][93]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[85][93]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[86][93]={3,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][93]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][93]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][93]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][93]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][93]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][93]={10,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][93]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][93]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][93]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][93]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[97][93]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][93]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][93]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][93]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][93]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][93]={5,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][93]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][93]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][93]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][93]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][93]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][93]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][93]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][93]={10,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][93]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][93]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][93]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][93]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][93]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][93]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][93]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[118][93]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][93]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][93]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][93]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][93]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][93]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][93]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][93]={3,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[126][93]={4,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][93]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[134][93]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][93]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[136][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][93]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][93]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][93]={1,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[173][93]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][93]={0,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][93]={2,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[176][93]={2,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][93]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][93]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][93]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][93]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][93]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][93]={3,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[183][93]={3,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[184][93]={3,3,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[185][93]={3,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][93]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[187][93]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][93]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][93]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][93]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][93]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][93]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][93]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][93]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][93]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][93]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][93]={9,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[198][93]={9,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[199][93]={9,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[200][93]={9,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[201][93]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[202][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][93]={9,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][93]={9,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[205][93]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][93]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][93]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][93]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][93]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][93]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][93]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][93]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][93]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][93]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][93]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][93]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][93]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][93]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][93]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][94]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][94]={0,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[7][94]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][94]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][94]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][94]={0,3,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[14][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][94]={1,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[26][94]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][94]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[31][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[35][94]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][94]={0,5,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][94]={0,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[39][94]={0,5,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[40][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][94]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][94]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][94]={3,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[44][94]={3,3,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[45][94]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][94]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][94]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][94]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][94]={0,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][94]={0,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[53][94]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][94]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][94]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][94]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][94]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][94]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][94]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][94]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][94]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][94]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][94]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][94]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][94]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][94]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][94]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][94]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][94]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][94]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][94]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][94]={3,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[74][94]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][94]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][94]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][94]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[79][94]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[80][94]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[81][94]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[82][94]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][94]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][94]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][94]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][94]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[92][94]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[93][94]={3,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][94]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][94]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][94]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][94]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][94]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][94]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][94]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][94]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][94]={11,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[105][94]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][94]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][94]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][94]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][94]={4,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[111][94]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][94]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][94]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][94]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][94]={4,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][94]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][94]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][94]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][94]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][94]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][94]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][94]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][94]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[125][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][94]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[134][94]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][94]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][94]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[150][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][94]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][94]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[172][94]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][94]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][94]={0,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][94]={1,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][94]={2,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][94]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][94]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][94]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][94]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][94]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][94]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][94]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][94]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][94]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][94]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][94]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][94]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[189][94]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][94]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][94]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][94]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][94]={10,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][94]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][94]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][94]={9,3,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[197][94]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][94]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][94]={9,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[200][94]={12,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[201][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][94]={9,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[204][94]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][94]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][94]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][94]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][94]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][94]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][94]={9,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][94]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][94]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][94]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][94]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][94]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][94]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][94]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[220][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[221][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][94]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][94]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[5][95]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][95]={1,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[7][95]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][95]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][95]={1,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[14][95]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][95]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][95]={0,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[27][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][95]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][95]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][95]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[32][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][95]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][95]={0,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][95]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][95]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][95]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][95]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][95]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][95]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][95]={0,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[45][95]={3,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[46][95]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[47][95]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[48][95]={3,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[49][95]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[50][95]={0,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[51][95]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][95]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][95]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][95]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][95]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][95]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][95]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][95]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][95]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][95]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][95]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][95]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][95]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][95]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][95]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][95]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][95]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][95]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][95]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][95]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][95]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][95]={9,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][95]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[74][95]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[75][95]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[76][95]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[77][95]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][95]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][95]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][95]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][95]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][95]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][95]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][95]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][95]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][95]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][95]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][95]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][95]={3,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][95]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][95]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][95]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][95]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][95]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][95]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][95]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][95]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][95]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][95]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][95]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][95]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[102][95]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][95]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][95]={10,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[105][95]={10,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[106][95]={3,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[107][95]={3,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[108][95]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[109][95]={3,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[110][95]={4,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[111][95]={4,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[112][95]={4,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[113][95]={4,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[114][95]={3,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[115][95]={3,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[116][95]={3,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][95]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][95]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][95]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][95]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][95]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][95]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][95]={0,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][95]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][95]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[134][95]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][95]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][95]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][95]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][95]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][95]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][95]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][95]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][95]={4,3,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[173][95]={4,3,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[174][95]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][95]={1,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[176][95]={1,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][95]={3,3,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[178][95]={3,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[179][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][95]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][95]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][95]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][95]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][95]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][95]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][95]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][95]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[188][95]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[189][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][95]={9,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[194][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][95]={9,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[197][95]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][95]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][95]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][95]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[203][95]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][95]={9,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][95]={9,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[208][95]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][95]={9,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][95]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][95]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][95]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[219][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][95]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][95]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[6][96]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][96]={0,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][96]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[13][96]={1,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][96]={0,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[15][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][96]={0,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[27][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][96]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[33][96]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[34][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[37][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][96]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][96]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][96]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][96]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[47][96]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][96]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][96]={9,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][96]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][96]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][96]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][96]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][96]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][96]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][96]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][96]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][96]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][96]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][96]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][96]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][96]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][96]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][96]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][96]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][96]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][96]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][96]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][96]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][96]={3,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[74][96]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][96]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][96]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][96]={3,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][96]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][96]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][96]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[95][96]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[96][96]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][96]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[102][96]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][96]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][96]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][96]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][96]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][96]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][96]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][96]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][96]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][96]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][96]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][96]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][96]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][96]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][96]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][96]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][96]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][96]={3,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][96]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][96]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][96]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][96]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[155][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][96]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][96]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][96]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][96]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][96]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][96]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][96]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][96]={5,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[175][96]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][96]={3,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[177][96]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][96]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][96]={3,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[180][96]={3,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[181][96]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][96]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][96]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][96]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][96]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][96]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][96]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][96]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][96]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][96]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][96]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][96]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][96]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][96]={12,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[195][96]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][96]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][96]={12,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[198][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[202][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][96]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][96]={12,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][96]={9,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[206][96]={9,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[207][96]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][96]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][96]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][96]={9,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[211][96]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][96]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][96]={12,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[214][96]={13,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[215][96]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][96]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][96]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[220][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][96]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][96]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][97]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][97]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][97]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][97]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][97]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[27][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][97]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][97]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][97]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][97]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[37][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][97]={0,5,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][97]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][97]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][97]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][97]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][97]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][97]={0,5,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][97]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][97]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][97]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][97]={9,3,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[50][97]={9,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[51][97]={9,-1,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[52][97]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][97]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][97]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][97]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][97]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][97]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][97]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][97]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][97]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][97]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][97]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][97]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][97]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][97]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][97]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][97]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][97]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][97]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][97]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][97]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][97]={3,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][97]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][97]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][97]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][97]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][97]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][97]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][97]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][97]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][97]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][97]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][97]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][97]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[90][97]={3,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[91][97]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[92][97]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[93][97]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][97]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][97]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][97]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][97]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[98][97]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][97]={3,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][97]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][97]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][97]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][97]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][97]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][97]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][97]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][97]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][97]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][97]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][97]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][97]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][97]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][97]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][97]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][97]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][97]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][97]={4,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][97]={11,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[124][97]={0,5,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][97]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][97]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[127][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][97]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][97]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][97]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][97]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][97]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][97]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[155][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[165][97]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][97]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][97]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][97]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][97]={4,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[171][97]={4,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[172][97]={4,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[173][97]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][97]={5,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[175][97]={4,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[176][97]={3,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[177][97]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][97]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][97]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][97]={9,3,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[181][97]={9,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[182][97]={3,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][97]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][97]={3,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][97]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][97]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][97]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][97]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][97]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][97]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][97]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][97]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][97]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][97]={12,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[195][97]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][97]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][97]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][97]={12,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][97]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][97]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][97]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][97]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][97]={9,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][97]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][97]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][97]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][97]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][97]={13,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[215][97]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][97]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][97]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[218][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][97]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][97]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[8][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[9][98]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[10][98]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[11][98]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][98]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][98]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[23][98]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][98]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][98]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[26][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][98]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][98]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][98]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][98]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][98]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][98]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][98]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][98]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][98]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][98]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][98]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][98]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][98]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][98]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][98]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][98]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][98]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][98]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][98]={9,3,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[53][98]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][98]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][98]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][98]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][98]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][98]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][98]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][98]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][98]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][98]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][98]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][98]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][98]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][98]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][98]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][98]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][98]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][98]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][98]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][98]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][98]={3,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[74][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][98]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][98]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][98]={9,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[78][98]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][98]={3,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][98]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][98]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][98]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][98]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][98]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][98]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][98]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][98]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][98]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][98]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][98]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][98]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][98]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][98]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][98]={3,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[102][98]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][98]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][98]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][98]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][98]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][98]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][98]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][98]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][98]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][98]={9,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[114][98]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][98]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][98]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][98]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][98]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][98]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][98]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][98]={11,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][98]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][98]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][98]={0,5,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[129][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][98]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[131][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[135][98]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][98]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][98]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[150][98]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][98]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][98]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[166][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][98]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][98]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][98]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][98]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][98]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][98]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][98]={4,3,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[174][98]={5,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[175][98]={4,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[176][98]={3,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[177][98]={3,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[178][98]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[179][98]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][98]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][98]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][98]={9,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[183][98]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][98]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][98]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][98]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][98]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][98]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][98]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][98]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][98]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][98]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][98]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][98]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[196][98]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][98]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[198][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][98]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][98]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][98]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][98]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][98]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][98]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][98]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][98]={9,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[211][98]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][98]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][98]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][98]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[215][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[216][98]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][98]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[219][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][98]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][98]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[9][99]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][99]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][99]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][99]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][99]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][99]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][99]={1,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][99]={1,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][99]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][99]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][99]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][99]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][99]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][99]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][99]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][99]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[37][99]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][99]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][99]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][99]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][99]={3,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[42][99]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][99]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][99]={0,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][99]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][99]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][99]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][99]={9,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][99]={9,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][99]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][99]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][99]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][99]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][99]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][99]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][99]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][99]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][99]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][99]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][99]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][99]={3,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][99]={9,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[78][99]={9,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][99]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][99]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][99]={3,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][99]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][99]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][99]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][99]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][99]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[97][99]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][99]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][99]={10,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[102][99]={3,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[103][99]={9,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[104][99]={9,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[105][99]={10,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[106][99]={3,3,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[107][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][99]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][99]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][99]={3,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][99]={9,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][99]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][99]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][99]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][99]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][99]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][99]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][99]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][99]={3,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][99]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][99]={0,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][99]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][99]={4,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[131][99]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][99]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][99]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][99]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[138][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[150][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][99]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][99]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][99]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][99]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][99]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[167][99]={3,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][99]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][99]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][99]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][99]={5,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][99]={5,-1,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[173][99]={4,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][99]={4,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][99]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][99]={3,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[177][99]={9,-1,17,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[178][99]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][99]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][99]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][99]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][99]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[183][99]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][99]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][99]={9,3,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[186][99]={10,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[187][99]={9,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[188][99]={9,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[189][99]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][99]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][99]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][99]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][99]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][99]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][99]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][99]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][99]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][99]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][99]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][99]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][99]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[210][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[211][99]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][99]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][99]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[215][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[216][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][99]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][99]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][99]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][100]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][100]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][100]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][100]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][100]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][100]={1,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][100]={1,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][100]={0,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][100]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][100]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[29][100]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][100]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][100]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][100]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[33][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][100]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][100]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][100]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][100]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][100]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][100]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][100]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][100]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][100]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][100]={3,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[45][100]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][100]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][100]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][100]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][100]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][100]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][100]={9,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][100]={9,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][100]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][100]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][100]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][100]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][100]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][100]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][100]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][100]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][100]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][100]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][100]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][100]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][100]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][100]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][100]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][100]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][100]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][100]={3,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[74][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][100]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][100]={9,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[79][100]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][100]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][100]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][100]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][100]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][100]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][100]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][100]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][100]={4,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[108][100]={4,3,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[109][100]={3,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[110][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][100]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][100]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][100]={3,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[114][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][100]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][100]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][100]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][100]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][100]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][100]={9,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][100]={9,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][100]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][100]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][100]={4,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][100]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][100]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[153][100]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][100]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[163][100]={10,3,17,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][100]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[165][100]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[166][100]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][100]={10,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][100]={10,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][100]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][100]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][100]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][100]={10,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][100]={10,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][100]={10,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][100]={9,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[176][100]={9,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[177][100]={9,3,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][100]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][100]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][100]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][100]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][100]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][100]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][100]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][100]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][100]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][100]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][100]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][100]={9,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[190][100]={12,-1,17,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[191][100]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][100]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[193][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[200][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[205][100]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][100]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][100]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][100]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][100]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][100]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][100]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][100]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][100]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][101]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][101]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][101]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][101]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][101]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][101]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[13][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][101]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][101]={0,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[22][101]={1,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[23][101]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][101]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][101]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][101]={0,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[27][101]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][101]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[29][101]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][101]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][101]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[33][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][101]={0,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][101]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][101]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][101]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][101]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][101]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][101]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[42][101]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][101]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][101]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][101]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][101]={9,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][101]={9,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][101]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][101]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][101]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][101]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][101]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][101]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][101]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][101]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][101]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][101]={9,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][101]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][101]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][101]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][101]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][101]={3,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][101]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][101]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][101]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][101]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][101]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][101]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][101]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][101]={4,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][101]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][101]={3,3,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[110][101]={10,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[111][101]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][101]={3,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][101]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][101]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][101]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][101]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][101]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][101]={11,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][101]={11,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][101]={3,3,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][101]={9,3,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][101]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][101]={9,-1,36,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[137][101]={9,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][101]={11,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][101]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][101]={11,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][101]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[142][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][101]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][101]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[151][101]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][101]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][101]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[156][101]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][101]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][101]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[162][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][101]={11,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[164][101]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][101]={11,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][101]={10,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[167][101]={10,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][101]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][101]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][101]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][101]={9,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][101]={9,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[173][101]={9,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[174][101]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][101]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][101]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][101]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][101]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][101]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][101]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][101]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][101]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][101]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][101]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][101]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][101]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][101]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][101]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][101]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][101]={12,-1,17,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[191][101]={12,-1,17,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[192][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[193][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][101]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][101]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[205][101]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][101]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][101]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[208][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[209][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][101]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][101]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][101]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][101]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][101]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][101]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][101]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][101]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][101]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][101]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][102]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][102]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[9][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[10][102]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][102]={5,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[12][102]={1,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][102]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[14][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][102]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][102]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][102]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][102]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][102]={4,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][102]={2,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[25][102]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][102]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][102]={0,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[28][102]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][102]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][102]={0,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][102]={0,5,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[32][102]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[35][102]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[36][102]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][102]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][102]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][102]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][102]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][102]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][102]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[43][102]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[44][102]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][102]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][102]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][102]={9,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][102]={9,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][102]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][102]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][102]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][102]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][102]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][102]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][102]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][102]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][102]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][102]={9,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[74][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][102]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][102]={9,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][102]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][102]={3,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[88][102]={9,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[89][102]={9,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[90][102]={11,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[91][102]={11,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[92][102]={10,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][102]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][102]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][102]={3,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][102]={3,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][102]={9,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[112][102]={3,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][102]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][102]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][102]={9,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][102]={11,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][102]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][102]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[144][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][102]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[146][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][102]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][102]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][102]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][102]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][102]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][102]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][102]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[161][102]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][102]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][102]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][102]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][102]={10,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][102]={10,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][102]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][102]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][102]={9,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][102]={9,3,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][102]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][102]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][102]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][102]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][102]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][102]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[179][102]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][102]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][102]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][102]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][102]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][102]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][102]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][102]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][102]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][102]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[203][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][102]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][102]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][102]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[213][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][102]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][102]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][102]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][102]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][102]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][102]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][103]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[10][103]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][103]={4,3,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[12][103]={4,3,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[13][103]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[14][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][103]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][103]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][103]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][103]={4,3,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[23][103]={4,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][103]={2,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][103]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][103]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][103]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][103]={1,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][103]={0,3,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[30][103]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][103]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][103]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[33][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][103]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][103]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][103]={0,-1,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[38][103]={0,5,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][103]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][103]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][103]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][103]={15,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][103]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][103]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][103]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][103]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][103]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][103]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][103]={12,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][103]={12,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][103]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][103]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][103]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][103]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][103]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][103]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][103]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][103]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][103]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][103]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][103]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][103]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][103]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][103]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][103]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][103]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][103]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][103]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][103]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][103]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][103]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][103]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][103]={9,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[74][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][103]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][103]={9,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][103]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][103]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][103]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][103]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][103]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][103]={9,-1,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[94][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][103]={9,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[112][103]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][103]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][103]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][103]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][103]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][103]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][103]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][103]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][103]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][103]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][103]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[151][103]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][103]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][103]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][103]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][103]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][103]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][103]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][103]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][103]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][103]={10,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][103]={10,3,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][103]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][103]={11,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][103]={10,3,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][103]={10,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][103]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][103]={10,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][103]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][103]={9,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][103]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][103]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][103]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][103]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][103]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][103]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][103]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][103]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][103]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][103]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][103]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][103]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][103]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][103]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][103]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][103]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[193][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][103]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][103]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[208][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][103]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[212][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[213][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][103]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[223][103]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[224][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[225][103]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[226][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][103]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][103]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[1][104]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[2][104]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][104]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][104]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][104]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[6][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][104]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][104]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[10][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[11][104]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][104]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][104]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][104]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][104]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[23][104]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][104]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][104]={4,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][104]={5,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][104]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][104]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][104]={1,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][104]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][104]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][104]={0,5,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][104]={0,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[34][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][104]={0,3,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][104]={0,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][104]={0,5,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[39][104]={3,3,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[40][104]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][104]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][104]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][104]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[45][104]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][104]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][104]={12,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[51][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][104]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][104]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][104]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][104]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][104]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][104]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][104]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][104]={9,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[75][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][104]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][104]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][104]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][104]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][104]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][104]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][104]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][104]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][104]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][104]={10,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[91][104]={9,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[92][104]={9,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[93][104]={9,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[94][104]={9,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][104]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][104]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][104]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][104]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][104]={9,-1,36,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][104]={9,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[106][104]={9,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[107][104]={9,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[108][104]={9,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[109][104]={10,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[110][104]={10,-1,36,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[111][104]={9,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][104]={9,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][104]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][104]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][104]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][104]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][104]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][104]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][104]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][104]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][104]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][104]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][104]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][104]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][104]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][104]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][104]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][104]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][104]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][104]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][104]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][104]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][104]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][104]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][104]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][104]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][104]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][104]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][104]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][104]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][104]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][104]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][104]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][104]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][104]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][104]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][104]={10,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[163][104]={10,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][104]={10,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[165][104]={11,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[166][104]={11,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[167][104]={9,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[168][104]={9,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[169][104]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][104]={9,3,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][104]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][104]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][104]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][104]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][104]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][104]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][104]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][104]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][104]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][104]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][104]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][104]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][104]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[194][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[200][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[207][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[208][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,0}}
 MapToConvert[209][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][104]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[211][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[212][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][104]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][104]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[220][104]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[221][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][104]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][104]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[224][104]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][104]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][104]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][104]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][104]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][105]={9,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[2][105]={3,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[3][105]={4,-1,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[4][105]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][105]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[6][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[11][105]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][105]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[13][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][105]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][105]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][105]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[23][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[24][105]={4,-1,33,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[25][105]={4,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][105]={4,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][105]={5,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][105]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][105]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][105]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][105]={3,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[32][105]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][105]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[36][105]={3,3,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][105]={3,3,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[38][105]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][105]={3,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[40][105]={3,-1,33,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][105]={3,3,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[42][105]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[44][105]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,0}}
 MapToConvert[45][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[46][105]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][105]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][105]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][105]={12,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,1,0}}
 MapToConvert[51][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][105]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][105]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][105]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][105]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][105]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][105]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][105]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[76][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[77][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][105]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][105]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][105]={12,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][105]={9,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[86][105]={9,3,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[87][105]={12,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[88][105]={12,-1,36,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[89][105]={10,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][105]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][105]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][105]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][105]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][105]={9,-1,36,{{1,2},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[104][105]={9,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][105]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][105]={10,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[111][105]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][105]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][105]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][105]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][105]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][105]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][105]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][105]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][105]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][105]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][105]={14,-1,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[142][105]={14,-1,36,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][105]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][105]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[147][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][105]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][105]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[151][105]={12,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[152][105]={12,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[153][105]={9,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[154][105]={9,-1,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[155][105]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][105]={11,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][105]={9,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[158][105]={9,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[159][105]={9,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[160][105]={9,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[161][105]={9,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[162][105]={9,3,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[163][105]={11,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][105]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][105]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][105]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][105]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][105]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][105]={12,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][105]={9,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][105]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][105]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][105]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][105]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[175][105]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][105]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][105]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[204][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[209][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][105]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[211][105]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][105]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[213][105]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[214][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][105]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][105]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][105]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[222][105]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][105]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][105]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][105]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][105]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][105]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[1][106]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][106]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][106]={14,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][106]={14,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][106]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][106]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][106]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][106]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][106]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[25][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[26][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[27][106]={4,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][106]={4,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][106]={5,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][106]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][106]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][106]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][106]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[37][106]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][106]={3,3,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[39][106]={3,-1,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[40][106]={3,3,33,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[41][106]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][106]={9,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,1,0}}
 MapToConvert[43][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[44][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[45][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][106]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][106]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][106]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[54][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[56][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][106]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][106]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[77][106]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][106]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][106]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][106]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][106]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][106]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][106]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][106]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][106]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][106]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][106]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][106]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][106]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][106]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][106]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][106]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][106]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][106]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][106]={9,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][106]={12,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][106]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][106]={11,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][106]={10,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][106]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][106]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][106]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][106]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][106]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][106]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][106]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][106]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][106]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][106]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][106]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][106]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[152][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][106]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][106]={14,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[156][106]={14,-1,17,{{1,5},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[157][106]={9,-1,17,{{1,5},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[158][106]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][106]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][106]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][106]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][106]={9,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][106]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][106]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][106]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][106]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][106]={9,3,17,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[171][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][106]={9,3,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[176][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][106]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[178][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[179][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[204][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[205][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][106]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][106]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[211][106]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][106]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][106]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][106]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][106]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][106]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][106]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][106]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][106]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][106]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][107]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][107]={14,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][107]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[27][107]={4,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][107]={3,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][107]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][107]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][107]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][107]={3,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][107]={3,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[34][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][107]={3,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[37][107]={3,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][107]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][107]={3,-1,33,{{0,-1},{1,3},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[40][107]={3,-1,33,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[41][107]={9,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][107]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[43][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[48][107]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[56][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[58][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[59][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[63][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[66][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[67][107]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][107]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[74][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[77][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[79][107]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][107]={9,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][107]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][107]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][107]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][107]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][107]={10,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][107]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][107]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][107]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][107]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][107]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][107]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][107]={10,3,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][107]={9,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][107]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][107]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][107]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][107]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][107]={9,3,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][107]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][107]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][107]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[145][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][107]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][107]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][107]={14,-1,17,{{0,-1},{0,-1},{1,4}},{-1,1},{0,0,0}}
 MapToConvert[158][107]={14,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][107]={11,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][107]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][107]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][107]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][107]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][107]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[165][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[166][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[167][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][107]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][107]={9,-1,17,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][107]={12,-1,17,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[184][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[191][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[204][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][107]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][107]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][107]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][107]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][107]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][107]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[214][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][107]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][107]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][107]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][107]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][107]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][107]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[227][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][107]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][107]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[1][108]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[15][108]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[16][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[28][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,0}}
 MapToConvert[29][108]={4,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][108]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][108]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][108]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][108]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][108]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[35][108]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][108]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][108]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][108]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][108]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][108]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][108]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][108]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][108]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][108]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][108]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[47][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[49][108]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][108]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][108]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][108]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][108]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][108]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][108]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][108]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][108]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[68][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[69][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][108]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[84][108]={9,3,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][108]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][108]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][108]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][108]={12,-1,36,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][108]={9,-1,36,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][108]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][108]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][108]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[119][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[120][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[121][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[133][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][108]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[154][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][108]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][108]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][108]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][108]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][108]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][108]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][108]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[179][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[190][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[191][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[197][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[198][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[205][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][108]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][108]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][108]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[210][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[211][108]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][108]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[218][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][108]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][108]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][108]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][108]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][108]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][108]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[1][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][109]={4,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[29][109]={4,3,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][109]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][109]={5,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][109]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][109]={3,3,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][109]={3,3,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][109]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][109]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][109]={3,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][109]={3,3,33,{{1,2},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[39][109]={10,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][109]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][109]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][109]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][109]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][109]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][109]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][109]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[47][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][109]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[50][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][109]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[67][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[84][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][109]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][109]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][109]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][109]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][109]={9,3,36,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[110][109]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[116][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][109]={9,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[122][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[125][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[129][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[130][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[133][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[134][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[136][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[141][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][109]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[143][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[154][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][109]={15,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[174][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[179][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][109]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][109]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][109]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][109]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][109]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][109]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][109]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[215][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[216][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[217][109]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][109]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][109]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][109]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][109]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][109]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[30][110]={4,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][110]={10,-1,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][110]={10,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][110]={11,-1,33,{{0,-1},{1,3},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][110]={10,-1,33,{{1,2},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][110]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][110]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][110]={3,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][110]={3,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][110]={9,3,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[40][110]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][110]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][110]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][110]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][110]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][110]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][110]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][110]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][110]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[51][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][110]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[58][110]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[62][110]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[63][110]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][110]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[73][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][110]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][110]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][110]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][110]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][110]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][110]={9,3,36,{{0,-1},{0,-1},{1,1}},{-1,1},{0,1,0}}
 MapToConvert[111][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[112][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[118][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[119][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[129][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[140][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[141][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[143][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[144][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][110]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][110]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[155][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[157][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[167][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[168][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][110]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][110]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][110]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][110]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][110]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][110]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][110]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][110]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[215][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[216][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[218][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][110]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][110]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][110]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[229][110]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][111]={9,3,33,{{0,-1},{1,0},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][111]={13,-1,33,{{1,5},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][111]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][111]={14,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][111]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][111]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][111]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][111]={3,3,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[38][111]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][111]={10,3,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[40][111]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][111]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][111]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][111]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][111]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][111]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][111]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][111]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][111]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[60][111]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[61][111]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][111]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][111]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][111]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[73][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[77][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[78][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[79][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][111]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][111]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][111]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][111]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][111]={13,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][111]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][111]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[100][111]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][111]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[104][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[106][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[110][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[111][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][111]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][111]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[157][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[160][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[161][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][111]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][111]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][111]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[204][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[205][111]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][111]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[208][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[209][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[215][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][111]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][111]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[229][111]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][112]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][112]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[31][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,0}}
 MapToConvert[32][112]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][112]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][112]={14,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][112]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][112]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][112]={11,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][112]={12,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[39][112]={10,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][112]={12,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][112]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][112]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][112]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][112]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][112]={9,3,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][112]={9,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][112]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][112]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][112]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][112]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][112]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][112]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[84][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][112]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][112]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[99][112]={15,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][112]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][112]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[107][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[109][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[123][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][112]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][112]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][112]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][112]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][112]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][112]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[158][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[159][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[163][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][112]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][112]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][112]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[174][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][112]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][112]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[186][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[187][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[194][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[195][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][112]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[201][112]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][112]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][112]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][112]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[206][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][112]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][112]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][112]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][112]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][112]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][112]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[0][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[32][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][113]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][113]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][113]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][113]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][113]={13,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][113]={12,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[39][113]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[40][113]={12,-1,33,{{0,-1},{1,0},{1,1}},{-1,1},{0,0,0}}
 MapToConvert[41][113]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[42][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[43][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[44][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][113]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][113]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][113]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][113]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[84][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[89][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[90][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[91][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[94][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[96][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][113]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][113]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][113]={14,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,1,1}}
 MapToConvert[106][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][113]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[128][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][113]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][113]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[174][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[178][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[181][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][113]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[200][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][113]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[203][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][113]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][113]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][113]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][113]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][113]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][113]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][113]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][113]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][113]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][113]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[0][114]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[34][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[35][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[36][114]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[37][114]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][114]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][114]={12,-1,33,{{0,-1},{0,-1},{1,1}},{-1,1},{0,1,0}}
 MapToConvert[40][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][114]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][114]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[64][114]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][114]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[69][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[70][114]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[93][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[95][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,1}}
 MapToConvert[97][114]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][114]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][114]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][114]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][114]={12,-1,36,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][114]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][114]={16,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[179][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,1,0}}
 MapToConvert[200][114]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[201][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][114]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][114]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][114]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][114]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][114]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][114]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][114]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][114]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][114]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][114]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][114]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][114]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[0][115]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[1][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[2][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[3][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[4][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[5][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[6][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[7][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[8][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[9][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[10][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[11][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[12][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[13][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[14][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[15][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[16][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[17][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[18][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[19][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[20][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[21][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[22][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[23][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[24][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[25][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[26][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[27][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[28][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[29][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[30][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[31][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[32][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[33][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[34][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[35][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[36][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[37][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[38][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[39][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[40][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[41][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[42][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[43][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[44][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[45][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[46][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[47][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[48][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[49][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[50][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[51][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[52][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[53][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[54][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[55][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[56][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[57][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[58][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[59][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[60][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[61][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[62][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[63][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[64][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[65][115]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[66][115]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[67][115]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[68][115]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,1}}
 MapToConvert[69][115]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[70][115]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[71][115]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[72][115]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[73][115]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[74][115]={12,-1,33,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[75][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[76][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[77][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[78][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[79][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[80][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[81][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[82][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[83][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[84][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[85][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[86][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[87][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[88][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[89][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[90][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[91][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[92][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[93][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[94][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[95][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[96][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[97][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[98][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[99][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[100][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[101][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[102][115]={15,-1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[103][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[104][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[105][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[106][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[107][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[108][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[109][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[110][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[111][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[112][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[113][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[114][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[115][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[116][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[117][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[118][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[119][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[120][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[121][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[122][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[123][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[124][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[125][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[126][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[127][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[128][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[129][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[130][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[131][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[132][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[133][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[134][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[135][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[136][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[137][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[138][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[139][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[140][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[141][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[142][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[143][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[144][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[145][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[146][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[147][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[148][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[149][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[150][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[151][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[152][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[153][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[154][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[155][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[156][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[157][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[158][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[159][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[160][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[161][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[162][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[163][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[164][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[165][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[166][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[167][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[168][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[169][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[170][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[171][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[172][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[173][115]={16,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[174][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[175][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[176][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[177][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[178][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[179][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[180][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[181][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[182][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[183][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[184][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[185][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[186][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[187][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[188][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[189][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[190][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[191][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[192][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[193][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[194][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[195][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[196][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[197][115]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[198][115]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[199][115]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{1,0,0}}
 MapToConvert[200][115]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,1}}
 MapToConvert[201][115]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[202][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[203][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[204][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[205][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[206][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[207][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[208][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[209][115]={15,1,-1,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[210][115]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[211][115]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[212][115]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[213][115]={14,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[214][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[215][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[216][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[217][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[218][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[219][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[220][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[221][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[222][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[223][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[224][115]={12,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[225][115]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[226][115]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[227][115]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[228][115]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}
 MapToConvert[229][115]={13,-1,17,{{0,-1},{0,-1},{0,-1}},{-1,1},{0,0,0}}

	
	return MapToConvert
end